@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 """
 Copyright (C) Jeremy O'Donoghue, 2003
- 
+
 License: This work is licensed under the PSF. A copy should be included
 with this source code, and is also available at
 http://www.python.org/psf/license.html
@@ -63,16 +63,16 @@ from matplotlib.backends.backend_wx import FigureCanvasWx,\
 
 from matplotlib.figure import Figure
 from matplotlib.axes import Subplot
-import matplotlib.numerix as numpy
-from wxPython.wx import *
+import  numpy
+from wx import *
 
 
-TIMER_ID = wxNewId()
+TIMER_ID = NewId()
 
-class PlotFigure(wxFrame):
+class PlotFigure(Frame):
 
     def __init__(self):
-        wxFrame.__init__(self, None, -1, "Test embedded wxFigure")
+        Frame.__init__(self, None, -1, "Test embedded wxFigure")
 
         self.fig = Figure((5,4), 75)
         self.canvas = FigureCanvasWx(self, -1, self.fig)
@@ -83,20 +83,20 @@ class PlotFigure(wxFrame):
         # you don't need this under Linux
         tw, th = self.toolbar.GetSizeTuple()
         fw, fh = self.canvas.GetSizeTuple()
-        self.toolbar.SetSize(wxSize(fw, th))
+        self.toolbar.SetSize(Size(fw, th))
 
         # Create a figure manager to manage things
         self.figmgr = FigureManager(self.canvas, 1, self)
         # Now put all into a sizer
-        sizer = wxBoxSizer(wxVERTICAL)
+        sizer = BoxSizer(VERTICAL)
         # This way of adding to sizer allows resizing
-        sizer.Add(self.canvas, 1, wxLEFT|wxTOP|wxGROW)
+        sizer.Add(self.canvas, 1, LEFT|TOP|GROW)
         # Best to allow the toolbar to resize!
-        sizer.Add(self.toolbar, 0, wxGROW)
+        sizer.Add(self.toolbar, 0, GROW)
         self.SetSizer(sizer)
         self.Fit()
         EVT_TIMER(self, TIMER_ID, self.onTimer)
-        
+
     def init_plot_data(self):
         a = self.fig.add_subplot(111)
         self.ind = numpy.arange(60)
@@ -108,26 +108,26 @@ class PlotFigure(wxFrame):
         self.count = 0
 
     def GetToolBar(self):
-        # You will need to override GetToolBar if you are using an 
+        # You will need to override GetToolBar if you are using an
         # unmanaged toolbar in your frame
         return self.toolbar
-		
+
     def onTimer(self, evt):
         self.count += 1
         if self.count >= 60: self.count = 0
         self.lines[0].set_data(self.ind, self.X[:,self.count])
         self.canvas.draw()
         self.canvas.gui_repaint()
-        
+
 if __name__ == '__main__':
-    app = wxPySimpleApp()
+    app = PySimpleApp()
     frame = PlotFigure()
     frame.init_plot_data()
-    
+
     # Initialise the timer - wxPython requires this to be connected to the
     # receivicng event handler
-    t = wxTimer(frame, TIMER_ID)
+    t = Timer(frame, TIMER_ID)
     t.Start(100)
-    
+
     frame.Show()
     app.MainLoop()
