@@ -3,29 +3,26 @@
 show how to add a matplotlib FigureCanvasGTK or FigureCanvasGTKAgg widget and
 a toolbar to a gtk.Window
 """
-
-from matplotlib.numerix import arange, sin, pi
-
-import matplotlib
-#matplotlib.use('GTK')
-matplotlib.use('GTKAgg')
-
-# switch comments for gtk over gtkagg
-#from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
-from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
-
-# or NavigationToolbar for classic
-from matplotlib.backends.backend_gtkagg import NavigationToolbar2GTKAgg as NavigationToolbar
+import gtk
 
 from matplotlib.axes import Subplot
 from matplotlib.figure import Figure
+from numpy import arange, sin, pi
+
+# uncomment to select /GTK/GTKAgg/GTKCairo
+from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
+#from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
+#from matplotlib.backends.backend_gtkcairo import FigureCanvasGTKCairo as FigureCanvas
+
+# or NavigationToolbar for classic
+from matplotlib.backends.backend_gtk import NavigationToolbar2GTK as NavigationToolbar
+#from matplotlib.backends.backend_gtkagg import NavigationToolbar2GTKAgg as NavigationToolbar
 
-import gtk
 
 win = gtk.Window()
+win.connect("destroy", lambda x: gtk.main_quit())
 win.set_default_size(400,300)
 win.set_title("Embedding in GTK")
-win.connect("destroy", lambda x: gtk.main_quit())
 
 vbox = gtk.VBox()
 win.add(vbox)
@@ -37,11 +34,12 @@ s = sin(2*pi*t)
 
 ax.plot(t,s)
 
+
 canvas = FigureCanvas(fig)  # a gtk.DrawingArea
 vbox.pack_start(canvas)
-
 toolbar = NavigationToolbar(canvas, win)
 vbox.pack_start(toolbar, False, False)
 
+
 win.show_all()
 gtk.main()
