@@ -10,11 +10,11 @@ figure(1, frameon=False)
 Z = arange(10000.0); Z.shape = 100,100
 Z[:,50:] = 1
 jet() # sets the default
-im1 = figimage(Z, xo=50,  yo=0)  
+im1 = figimage(Z, xo=50,  yo=0)
 im2 = figimage(Z, xo=100, yo=100, alpha=.8)
 #gray()  # overrides current and sets default
-#savefig('figimage_demo')
+savefig('figimage_demo')
 
 show()
 
-    
+
