@@ -3,7 +3,7 @@ from pylab import *
 
 w, h = 512, 512
 s = file('data/ct.raw', 'rb').read()
-A = fromstring(s, UInt16).astype(Float)
+A = fromstring(s, uint16).astype(float)
 A *= 1.0/max(A)
 A.shape = w, h
 
