@@ -3,7 +3,7 @@ from pylab import *
 
 # data are 256x256 16 bit integers
 dfile = 'data/s1045.ima'
-im = fromstring(file(dfile, 'rb').read(), UInt16).astype(Float)
+im = fromstring(file(dfile, 'rb').read(), uint16).astype(float)
 im.shape = 256, 256
 
 #imshow(im, ColormapJet(256))
