@@ -5,7 +5,7 @@ using imshow that is likely faster for large grids
 """
 from __future__ import division
 from matplotlib.patches import Patch
-from matplotlib.matlab import *
+from pylab import *
 
 def func3(x,y):
     return (1- x/2 + x**5 + y**3)*exp(-x**2-y**2)
@@ -19,10 +19,10 @@ y = arange(-3.0, 3.0001, dy)
 X,Y = meshgrid(x, y)
 
 Z = func3(X, Y)
-pcolor(X, Y, Z, shading='flat')
+pcolor(X, Y, Z)
 colorbar()
 axis([-3,3,-3,3])
-#savefig('pcolor_demo')
+savefig('pcolor_demo')
 show()
 
-    
+
