@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
-import matplotlib.numerix as nx
-from pylab import figure, show, rc
+import numpy as npy
+from matplotlib.pyplot import figure, show, rc
 
 # radar green, solid grid lines
 rc('grid', color='#316931', linewidth=1, linestyle='-')
@@ -12,8 +12,8 @@ rc('ytick', labelsize=15)
 fig = figure(figsize=(8,8))
 ax = fig.add_axes([0.1, 0.1, 0.8, 0.8], polar=True, axisbg='#d5de9c')
 
-r = nx.arange(0, 3.0, 0.01)
-theta = 2*nx.pi*r
+r = npy.arange(0, 3.0, 0.01)
+theta = 2*npy.pi*r
 ax.plot(theta, r, color='#ee8d18', lw=3, label='a line')
 ax.plot(0.5*theta, r, color='blue', ls='--', lw=3, label='another line')
 ax.legend()
