@@ -55,9 +55,9 @@ gmail.com and a host of others).
 """
 from __future__ import generators
 
-__version__  = '0.91.1'
-__revision__ = '$Revision: 4514 $'
-__date__     = '$Date: 2007-11-29 21:47:06 -0500 (Thu, 29 Nov 2007) $'
+__version__  = '0.91.2'
+__revision__ = '$Revision: 4802 $'
+__date__     = '$Date: 2008-01-06 12:10:34 -0500 (Sun, 6 Jan 2008) $'
 
 import md5, os, re, shutil, sys, warnings
 import distutils.sysconfig
@@ -428,11 +428,16 @@ def _get_data_path():
 
     # py2exe zips pure python, so still need special check
     if getattr(sys,'frozen',None):
-        path = os.path.join(os.path.split(sys.path[0])[0], 'matplotlibdata')
+        path = os.path.join(os.path.split(sys.path[0])[0], 'mpl-data')
+        if os.path.isdir(path): return path
+        else:
+            # Try again assuming we need to step up one more directory
+            path = os.path.join(os.path.split(os.path.split(sys.path[0])[0])[0],
+                                'mpl-data')
         if os.path.isdir(path): return path
         else:
             # Try again assuming sys.path[0] is a dir not a exe
-            path = os.path.join(sys.path[0], 'matplotlibdata')
+            path = os.path.join(sys.path[0], 'mpl-data')
             if os.path.isdir(path): return path
 
     raise RuntimeError('Could not find the matplotlib data files')
@@ -442,7 +447,8 @@ def _get_data_path_cached():
         defaultParams['datapath'][0] = _get_data_path()
     return defaultParams['datapath'][0]
 
-get_data_path = verbose.wrap('matplotlib data path %s', _get_data_path_cached, always=False)
+get_data_path = verbose.wrap('matplotlib data path %s', _get_data_path_cached,
+                             always=False)
 
 def get_py2exe_datafiles():
     datapath = get_data_path()
@@ -454,8 +460,8 @@ def get_py2exe_datafiles():
         if 'Matplotlib.nib' in files:
             files.remove('Matplotlib.nib')
         files = [os.path.join(root, filename) for filename in files]
-        root = root.replace(tail, 'matplotlibdata')
-        root = root[root.index('matplotlibdata'):]
+        root = root.replace(tail, 'mpl-data')
+        root = root[root.index('mpl-data'):]
         d[root] = files
     return d.items()
 
@@ -615,6 +621,15 @@ distribution""" % (key, cnt, fname)
     if ret['datapath'] is None:
         ret['datapath'] = get_data_path()
 
+    if not ret['text.latex.preamble'] == ['']:
+        verbose.report("""
+*****************************************************************
+You have the following UNSUPPORTED LaTeX preamble customizations:
+%s
+Please do not ask for support with these customizations active.
+*****************************************************************
+"""% '\n'.join(ret['text.latex.preamble']), 'helpful')
+
     verbose.report('loaded rc file %s'%fname)
 
     return ret
@@ -712,8 +727,11 @@ if NEWCONFIG:
     except:
         from config import rcParams, rcdefaults
 
-_use_error_msg = """ matplotlib.use() must be called *before* pylab
-or matplotlib.backends is imported for the first time."""
+_use_error_msg = """ This call to matplotlib.use() has no effect
+because the the backend has already been chosen;
+matplotlib.use() must be called *before* pylab, matplotlib.pyplot,
+or matplotlib.backends is imported for the first time.
+"""
 
 def use(arg):
     """
@@ -732,7 +750,7 @@ def use(arg):
     be called before importing matplotlib.backends.
     """
     if 'matplotlib.backends' in sys.modules:
-        raise RuntimeError(_use_error_msg)
+        warnings.warn(_use_error_msg)
     be_parts = arg.split('.')
     name = validate_backend(be_parts[0])
     rcParams['backend'] = name
@@ -784,60 +802,3 @@ verbose.report('units is %s'%rcParams['units'])
 verbose.report('platform is %s'%sys.platform)
 verbose.report('loaded modules: %s'%sys.modules.keys(), 'debug')
 
-class ExampleInfo:
-    pass
-
-class ExampleManager:
-    baseurl = 'http://matplotlib.sf.net'
-    urls = ['%s/%s'%(baseurl, subdir) for subdir in
-            ( 'examples', 'examples/widgets')]
-
-    def get_examples(self):
-        import urllib, re
-        rgx = re.compile('.*<A HREF="([^.]+\.py)">.*')
-        examples = []
-        for url in urls:
-            lines = urllib.urlopen(url).readlines()
-            for line in lines:
-                m = rgx.match(line)
-                if m is not None:
-                    examples.append('%s/%s'%(url, m.group(1)))
-        return examples
-
-    def get_info(self, s):
-        """
-        return an ExampleInfo instance from s, the string content of
-        an example
-        """
-        pass
-
-class Namespace:
-    """
-    A class which takes a list of modules and creates an object with
-    the module naems at attrs
-    """
-    def __init__(self, namespace):
-        for k,v in namespace.items():
-            modname = getattr(v, '__name__', None)
-            if modname is None: continue
-            if modname.startswith('matplotlib.'):
-                self.__dict__[modname.replace('matplotlib.', '')] = v
-
-
-class Importer:
-    def __init__(self, modstr):
-        """
-        import a bunch of matplotlib modules listed in modstr into a
-        single namespace.  Eg,
-
-        mpl = Importer('artist, cbook, lines, patches')
-        print mpl.cbook.iterable(1)
-        """
-        for name in modstr.split(','):
-            name = name.strip()
-            wholename = '.'.join(['matplotlib', name])
-            basemod = __import__(wholename)
-            mod = getattr(basemod, name)
-            setattr(self, name, mod)
-
-
