@@ -1,8 +1,9 @@
 from __future__ import division
-import sys
+import sys, re
 from cbook import iterable, flatten
 from transforms import identity_transform
-import warnings
+import matplotlib.units as units
+
 ## Note, matplotlib artists use the doc strings for set and get
 # methods to enable the introspection methods of setp and getp.  Every
 # set_* method should have a docstring containing the line
@@ -30,19 +31,87 @@ class Artist:
     def __init__(self):
         self.figure = None
 
-        self._transform = identity_transform()
+        self._transform = None
         self._transformSet = False
         self._visible = True
         self._animated = False
         self._alpha = 1.0
         self.clipbox = None
+        self._clippath = None
         self._clipon = False
         self._lod = False
         self._label = ''
+        self._picker = None
+        self._contains = None
 
         self.eventson = False  # fire events only if eventson
         self._oid = 0  # an observer id
         self._propobservers = {} # a dict from oids to funcs
+        self.axes = None
+        self._remove_method = None
+
+    def remove(self):
+        """
+        Remove the artist from the figure if possible.  The effect will not
+        be visible until the figure is redrawn, e.g., with ax.draw_idle().
+        Call ax.relim() to update the axes limits if desired.
+
+        Note: relim() will not see collections even if the collection
+        was added to axes with autolim=True.
+
+        Note: there is no support for removing the artist's legend entry.
+        """
+
+        # There is no method to set the callback.  Instead the parent should set
+        # the _remove_method attribute directly.  This would be a protected
+        # attribute if Python supported that sort of thing.  The callback
+        # has one parameter, which is the child to be removed.
+        if self._remove_method != None:
+            self._remove_method(self)
+        else:
+            raise NotImplementedError('cannot remove artist')
+        # TODO: the fix for the collections relim problem is to move the
+        # limits calculation into the artist itself, including the property
+        # of whether or not the artist should affect the limits.  Then there
+        # will be no distinction between axes.add_line, axes.add_patch, etc.
+        # TODO: add legend support
+
+    def have_units(self):
+        'return True if units are set on the x or y axes'
+        ax = self.axes
+        if ax is None or ax.xaxis is None:
+            return False
+        return ax.xaxis.have_units() or ax.yaxis.have_units()
+
+    def convert_xunits(self, x):
+        """for artists in an axes, if the xaxis as units support,
+        convert x using xaxis unit type
+        """
+        ax = getattr(self, 'axes', None)
+        if ax is None or ax.xaxis is None:
+            #print 'artist.convert_xunits no conversion: ax=%s'%ax
+            return x
+        return ax.xaxis.convert_units(x)
+
+    def convert_yunits(self, y):
+        """for artists in an axes, if the yaxis as units support,
+        convert y using yaxis unit type
+        """
+        ax = getattr(self, 'axes', None)
+        if ax is None or ax.yaxis is None: return y
+        return ax.yaxis.convert_units(y)
+
+    def set_axes(self, axes):
+        """
+        set the axes instance the artist resides in, if any
+
+        ACCEPTS: an axes instance
+        """
+        self.axes = axes
+
+    def get_axes(self):
+        'return the axes instance the artist resides in, or None'
+        return self.axes
 
     def add_callback(self, func):
         oid = self._oid
@@ -75,8 +144,120 @@ class Artist:
 
     def get_transform(self):
         'return the Transformation instance used by this artist'
+        if self._transform is None:
+            self._transform = identity_transform()
         return self._transform
 
+    def hitlist(self,event):
+        """List the children of the artist which contain the mouse event"""
+        import traceback
+        L = []
+        try:
+            hascursor,info = self.contains(event)
+            if hascursor:
+                L.append(self)
+        except:
+            traceback.print_exc()
+            print "while checking",self.__class__
+
+        if hasattr(self,'get_children'):
+            for a in self.get_children(): L.extend(a.hitlist(event))
+        return L
+
+    def contains(self,mouseevent):
+        """Test whether the artist contains the mouse event.
+
+        Returns the truth value and a dictionary of artist specific details of
+        selection, such as which points are contained in the pick radius.  See
+        individual artists for details.
+        """
+        if callable(self._contains): return self._contains(self,mouseevent)
+        #raise NotImplementedError,str(self.__class__)+" needs 'contains' method"
+        print str(self.__class__)+" needs 'contains' method"
+        return False,{}
+
+    def set_contains(self,picker):
+        """Replace the contains test used by this artist. The new picker should
+        be a callable function which determines whether the artist is hit by the
+        mouse event:
+
+            hit, props = picker(artist, mouseevent)
+
+        If the mouse event is over the artist, return hit=True and props
+        is a dictionary of properties you want returned with the contains test.
+        """
+        self._contains = picker
+
+    def get_contains(self):
+        'return the _contains test used by the artist, or None for default.'
+        return self._contains
+
+    def pickable(self):
+        'return True if self is pickable'
+        return (self.figure is not None and
+                self.figure.canvas is not None and
+                self._picker is not None)
+
+    def pick(self, mouseevent):
+        """
+        pick(mouseevent)
+
+        each child artist will fire a pick event if mouseevent is over
+        the artist and the artist has picker set
+        """
+        # Pick self
+        if self.pickable():
+            picker = self.get_picker()
+            if callable(picker):
+                inside,prop = picker(self,mouseevent)
+            else:
+                inside,prop = self.contains(mouseevent)
+            if inside:
+                self.figure.canvas.pick_event(mouseevent, self, **prop)
+
+        # Pick children
+        if hasattr(self,'get_children'):
+            for a in self.get_children(): a.pick(mouseevent)
+
+    def set_picker(self, picker):
+        """
+        set the epsilon for picking used by this artist
+
+        picker can be one of the following:
+
+          None -  picking is disabled for this artist (default)
+
+          boolean - if True then picking will be enabled and the
+            artist will fire a pick event if the mouse event is over
+            the artist
+
+          float - if picker is a number it is interpreted as an
+            epsilon tolerance in points and the the artist will fire
+            off an event if it's data is within epsilon of the mouse
+            event.  For some artists like lines and patch collections,
+            the artist may provide additional data to the pick event
+            that is generated, eg the indices of the data within
+            epsilon of the pick event
+
+          function - if picker is callable, it is a user supplied
+            function which determines whether the artist is hit by the
+            mouse event.
+
+              hit, props = picker(artist, mouseevent)
+
+            to determine the hit test.  if the mouse event is over the
+            artist, return hit=True and props is a dictionary of
+            properties you want added to the PickEvent attributes
+
+        ACCEPTS: [None|float|boolean|callable]
+        """
+        self._picker = picker
+
+    def get_picker(self):
+        'return the Pickeration instance used by this artist'
+        return self._picker
+
+
     def is_figure_set(self):
         return self.figure is not None
 
@@ -100,9 +281,20 @@ class Artist:
         ACCEPTS: a matplotlib.transform.Bbox instance
         """
         self.clipbox = clipbox
-        self._clipon = clipbox is not None
+        self._clipon = clipbox is not None or self._clippath is not None
         self.pchanged()
 
+    def set_clip_path(self, path):
+        """
+        Set the artist's clip path
+
+        ACCEPTS: an agg.path_storage instance
+        """
+        self._clippath = path
+        self._clipon = self.clipbox is not None or path is not None
+        self.pchanged()
+
+
     def get_alpha(self):
         """
         Return the alpha value used for blending - not supported on all
@@ -120,12 +312,16 @@ class Artist:
 
     def get_clip_on(self):
         'Return whether artist uses clipping'
-        return self._clipon and self.clipbox is not None
+        return self._clipon and (self.clipbox is not None or self._clippath is not None)
 
     def get_clip_box(self):
         'Return artist clipbox'
         return self.clipbox
 
+    def get_clip_path(self):
+        'Return artist clip path'
+        return self._clippath
+
     def set_clip_on(self, b):
         """
         Set  whether artist uses clipping
@@ -133,9 +329,17 @@ class Artist:
         ACCEPTS: [True | False]
         """
         self._clipon = b
-        if not b: self.clipbox = None
+        if not b:
+            self.clipbox = None
+            self._clippath = None
         self.pchanged()
 
+    def _set_gc_clip(self, gc):
+        'set the clip properly for the gc'
+        if self.clipbox is not None:
+            gc.set_clip_rectangle(self.clipbox.get_bounds())
+        gc.set_clip_path(self._clippath)
+
     def draw(self, renderer, *args, **kwargs):
         'Derived classes drawing method'
         if not self.get_visible(): return
@@ -206,6 +410,8 @@ class Artist:
         self._label = s
         self.pchanged()
 
+
+
     def get_zorder(self): return self.zorder
 
     def set_zorder(self, level):
@@ -242,9 +448,10 @@ class Artist:
             ret.extend( [func(v)] )
         return ret
 
+
 class ArtistInspector:
     """
-    A helper class to insect an Artist and return information about
+    A helper class to inspect an Artist and return information about
     it's settable properties and their current values
     """
     def __init__(self, o):
@@ -277,6 +484,7 @@ class ArtistInspector:
             aliases[fullname[4:]] = name[4:]
         return aliases
 
+    _get_valid_values_regex = re.compile(r"\n\s*ACCEPTS:\s*(.*)\n")
     def get_valid_values(self, attr):
         """
         get the legal arguments for the setter associated with attr
@@ -298,10 +506,10 @@ class ArtistInspector:
 
         if docstring.startswith('alias for '):
             return None
-        for line in docstring.split('\n'):
-            line = line.lstrip()
-            if not line.startswith('ACCEPTS:'): continue
-            return line[8:].strip()
+
+        match = self._get_valid_values_regex.search(docstring)
+        if match is not None:
+            return match.group(1)
         return 'unknown'
 
     def get_setters(self):
@@ -339,7 +547,7 @@ class ArtistInspector:
             return '%s or %s' % (s, self.aliasd[s])
         else: return s
 
-    def pprint_setters(self, prop=None):
+    def pprint_setters(self, prop=None, leadingspace=2):
         """
         if prop is None, return a list of strings of all settable properies
         and their valid values
@@ -348,9 +556,13 @@ class ArtistInspector:
         property will be returned as a string of property : valid
         values
         """
+        if leadingspace:
+            pad = ' '*leadingspace
+        else:
+            pad  = ''
         if prop is not None:
             accepts = self.get_valid_values(prop)
-            return '    %s: %s' %(prop, accepts)
+            return '%s%s: %s' %(pad, prop, accepts)
 
         attrs = self.get_setters()
         attrs.sort()
@@ -359,7 +571,8 @@ class ArtistInspector:
         for prop in attrs:
             accepts = self.get_valid_values(prop)
             name = self.aliased_name(prop)
-            lines.append('    %s: %s' %(name, accepts))
+
+            lines.append('%s%s: %s' %(pad, name, accepts))
         return lines
 
     def pprint_getters(self):
@@ -376,7 +589,7 @@ class ArtistInspector:
             if self.is_alias(func): continue
             try: val = func()
             except: continue
-            if hasattr(val, 'shape') and len(val)>6:
+            if getattr(val, 'shape', ()) != () and len(val)>6:
                 s = str(val[:6]) + '...'
             else:
                 s = str(val)
@@ -423,9 +636,9 @@ get.__doc__ = getp.__doc__
 
 def setp(h, *args, **kwargs):
     """
-    matplotlib supports the use of setp ("set property") and getp to set 
-    and get object properties, as well as to do introspection on the 
-    object For example, to set the linestyle of a line to be dashed, you 
+    matplotlib supports the use of setp ("set property") and getp to set
+    and get object properties, as well as to do introspection on the
+    object For example, to set the linestyle of a line to be dashed, you
     can do
 
       >>> line, = plot([1,2,3])
@@ -494,6 +707,8 @@ def setp(h, *args, **kwargs):
             ret.extend( [func(val)] )
     return [x for x in flatten(ret)]
 
+def kwdoc(a):
+    return '\n'.join(ArtistInspector(a).pprint_setters(leadingspace=4))
 
-
-
+kwdocd = dict()
+kwdocd['Artist'] = kwdoc(Artist)
