@@ -214,7 +214,7 @@ def is_string_like(obj):
     return 1
 
 def is_writable_file_like(obj):
-    return hasattr(filename, 'write') and callable(filename.write)
+    return hasattr(obj, 'write') and callable(obj.write)
 
 def is_scalar(obj):
     return is_string_like(obj) or not iterable(obj)
@@ -224,7 +224,7 @@ def is_numlike(obj):
     except TypeError: return False
     else: return True
 
-def to_filehandle(fname, flag='r'):
+def to_filehandle(fname, flag='r', return_opened=False):
     """
     fname can be a filename or a file handle.  Support for gzipped
     files is automatic, if the filename ends in .gz.  flag is a
@@ -236,10 +236,14 @@ def to_filehandle(fname, flag='r'):
             fh = gzip.open(fname, flag)
         else:
             fh = file(fname, flag)
+        opened = True
     elif hasattr(fname, 'seek'):
         fh = fname
+        opened = False
     else:
         raise ValueError('fname must be a string or file handle')
+    if return_opened:
+        return fh, opened
     return fh
 
 def flatten(seq, scalarp=is_scalar):
@@ -891,7 +895,7 @@ class MemoryMonitor:
         return x, self._mem[i0:self._n:isub]
 
     def plot(self, i0=0, isub=1, fig=None):
-        if fig is None:            
+        if fig is None:
             from pylab import figure, show
             fig = figure()
 
@@ -962,6 +966,10 @@ def print_cycles(objects, outstream=sys.stdout, show_progress=False):
         outstream.write("Examining: %r\n" % (obj,))
         recurse(obj, obj, { }, [])
 
+
+
+
+
 if __name__=='__main__':
     assert( allequal([1,1,1]) )
     assert(not  allequal([1,1,0]) )
