@@ -4,7 +4,7 @@ operations.
 
 """
 from __future__ import division
-import sys, os
+import sys, os, warnings
 
 import numpy as npy
 
@@ -23,6 +23,7 @@ from matplotlib import _image
 from matplotlib._image import *
 
 class AxesImage(martist.Artist, cm.ScalarMappable):
+    zorder = 1
 
     def __init__(self, ax,
                  cmap = None,
@@ -185,6 +186,9 @@ class AxesImage(martist.Artist, cm.ScalarMappable):
 
     def draw(self, renderer, *args, **kwargs):
         if not self.get_visible(): return
+        if (self.axes.get_xscale() != 'linear' or
+            self.axes.get_yscale() != 'linear'):
+            warnings.warn("Images are not supported on non-linear axes.")
         im = self.make_image(renderer.get_image_magnification())
         l, b, widthDisplay, heightDisplay = self.axes.bbox.get_bounds()
         renderer.draw_image(l, b, im, self.axes.bbox)
@@ -243,6 +247,8 @@ class AxesImage(martist.Artist, cm.ScalarMappable):
 
         self.set_data(A)
 
+
+
     def set_extent(self, extent):
         """extent is data axes (left, right, bottom, top) for making image plots
         """
@@ -512,18 +518,21 @@ class PcolorImage(martist.Artist, cm.ScalarMappable):
         self.update_dict['array'] = True
 
 class FigureImage(martist.Artist, cm.ScalarMappable):
+    zorder = 1
     def __init__(self, fig,
                  cmap = None,
                  norm = None,
                  offsetx = 0,
                  offsety = 0,
                  origin=None,
+                 **kwargs
                  ):
 
         """
         cmap is a colors.Colormap instance
         norm is a colors.Normalize instance to map luminance to 0-1
 
+        kwargs are an optional list of Artist keyword args
         """
         martist.Artist.__init__(self)
         cm.ScalarMappable.__init__(self, norm, cmap)
@@ -532,6 +541,7 @@ class FigureImage(martist.Artist, cm.ScalarMappable):
         self.figure = fig
         self.ox = offsetx
         self.oy = offsety
+        self.update(kwargs)
 
     def contains(self, mouseevent):
         """Test whether the mouse event occured within the image.
