@@ -8,11 +8,9 @@ from __future__ import division
 
 import sys, math, warnings
 
+import numpy as npy
+
 import agg
-from numerix import Float, alltrue, arange, array, logical_and,\
-     nonzero, searchsorted, take, asarray, ones, where, less, ravel, \
-     greater, logical_and, cos, sin, pi, sqrt, less_equal, \
-     compress, zeros, concatenate, cumsum, typecode, NewAxis
 import numerix.ma as ma
 from matplotlib import verbose
 import artist
@@ -22,58 +20,99 @@ from colors import colorConverter
 
 from transforms import lbwh_to_bbox, LOG10
 from matplotlib import rcParams
-TICKLEFT, TICKRIGHT, TICKUP, TICKDOWN = range(4)
+
+# special-purpose marker identifiers:
+(TICKLEFT, TICKRIGHT, TICKUP, TICKDOWN,
+    CARETLEFT, CARETRIGHT, CARETUP, CARETDOWN) = range(8)
 
 def unmasked_index_ranges(mask, compressed = True):
     '''
-    Calculate the good data ranges in a masked 1-D array, based on mask.
+    Calculate the good data ranges in a masked 1-D npy.array, based on mask.
 
-    Returns Nx2 array with each row the start and stop indices
-    for slices of the compressed array corresponding to each of N
+    Returns Nx2 npy.array with each row the start and stop indices
+    for slices of the compressed npy.array corresponding to each of N
     uninterrupted runs of unmasked values.
     If optional argument compressed is False, it returns the
-    start and stop indices into the original array, not the
-    compressed array.
+    start and stop indices into the original npy.array, not the
+    compressed npy.array.
     Returns None if there are no unmasked values.
 
     Example:
 
-    y = ma.array(arange(5), mask = [0,0,1,0,0])
+    y = ma.array(npy.arange(5), mask = [0,0,1,0,0])
     #ii = unmasked_index_ranges(y.mask())
     ii = unmasked_index_ranges(ma.getmask(y))
         # returns [[0,2,] [2,4,]]
 
     y.compressed().filled()[ii[1,0]:ii[1,1]]
-        # returns array [3,4,]
-        # (The 'filled()' method converts the masked array to a numerix array.)
+        # returns npy.array [3,4,]
+        # (The 'filled()' method converts the masked npy.array to a numerix npy.array.)
 
     #i0, i1 = unmasked_index_ranges(y.mask(), compressed=False)
     i0, i1 = unmasked_index_ranges(ma.getmask(y), compressed=False)
         # returns [[0,3,] [2,5,]]
 
     y.filled()[ii[1,0]:ii[1,1]]
-        # returns array [3,4,]
+        # returns npy.array [3,4,]
 
     '''
-    m = concatenate(((1,), mask, (1,)))
-    indices = arange(len(mask) + 1)
+    m = npy.concatenate(((1,), mask, (1,)))
+    indices = npy.arange(len(mask) + 1)
     mdif = m[1:] - m[:-1]
-    i0 = compress(mdif == -1, indices)
-    i1 = compress(mdif == 1, indices)
+    i0 = npy.compress(mdif == -1, indices)
+    i1 = npy.compress(mdif == 1, indices)
     assert len(i0) == len(i1)
     if len(i1) == 0:
         return None
     if not compressed:
-        return concatenate((i0[:, NewAxis], i1[:, NewAxis]), axis=1)
+        return npy.concatenate((i0[:, npy.newaxis], i1[:, npy.newaxis]), axis=1)
     seglengths = i1 - i0
-    breakpoints = cumsum(seglengths)
-    ic0 = concatenate(((0,), breakpoints[:-1]))
+    breakpoints = npy.cumsum(seglengths)
+    ic0 = npy.concatenate(((0,), breakpoints[:-1]))
     ic1 = breakpoints
-    return concatenate((ic0[:, NewAxis], ic1[:, NewAxis]), axis=1)
+    return npy.concatenate((ic0[:, npy.newaxis], ic1[:, npy.newaxis]), axis=1)
 
+def segment_hits(cx,cy,x,y,radius):
+    """Determine if any line segments are within radius of a point. Returns
+    the list of line segments that are within that radius.
+    """
+    # Process single points specially
+    if len(x) < 2:
+        res, = npy.nonzero( (cx - x)**2 + (cy - y)**2 <= radius**2 )
+        return res
+
+    # We need to lop the last element off a lot.
+    xr,yr = x[:-1],y[:-1]
+
+    # Only look at line segments whose nearest point to C on the line
+    # lies within the segment.
+    dx,dy = x[1:]-xr, y[1:]-yr
+    Lnorm_sq = dx**2+dy**2    # Possibly want to eliminate Lnorm==0
+    u = ( (cx-xr)*dx + (cy-yr)*dy )/Lnorm_sq
+    candidates = (u>=0) & (u<=1)
+    #if any(candidates): print "candidates",xr[candidates]
+
+    # Note that there is a little area near one side of each point
+    # which will be near neither segment, and another which will
+    # be near both, depending on the angle of the lines.  The
+    # following radius test eliminates these ambiguities.
+    point_hits = (cx - x)**2 + (cy - y)**2 <= radius**2
+    #if any(point_hits): print "points",xr[candidates]
+    candidates = candidates & ~point_hits[:-1] & ~point_hits[1:]
+
+    # For those candidates which remain, determine how far they lie away
+    # from the line.
+    px,py = xr+u*dx,yr+u*dy
+    line_hits = (cx-px)**2 + (cy-py)**2 <= radius**2
+    #if any(line_hits): print "lines",xr[candidates]
+    line_hits = line_hits & candidates
+    points, = point_hits.ravel().nonzero()
+    lines, = line_hits.ravel().nonzero()
+    #print points,lines
+    return npy.concatenate((points,lines))
 
 class Line2D(Artist):
-    _lineStyles =  {
+    lineStyles = _lineStyles =  { # hidden names deprecated
         '-'    : '_draw_solid',
         '--'   : '_draw_dashed',
         '-.'   : '_draw_dash_dot',
@@ -84,7 +123,7 @@ class Line2D(Artist):
         ''     : '_draw_nothing',
     }
 
-    _markers =  {
+    markers = _markers =  {  # hidden names deprecated
         '.'  : '_draw_point',
         ','  : '_draw_pixel',
         'o'  : '_draw_circle',
@@ -110,6 +149,10 @@ class Line2D(Artist):
         TICKRIGHT   : '_draw_tickright',
         TICKUP      : '_draw_tickup',
         TICKDOWN    : '_draw_tickdown',
+        CARETLEFT   : '_draw_caretleft',
+        CARETRIGHT  : '_draw_caretright',
+        CARETUP     : '_draw_caretup',
+        CARETDOWN   : '_draw_caretdown',
         'None' : '_draw_nothing',
         ' ' : '_draw_nothing',
         '' : '_draw_nothing',
@@ -121,6 +164,16 @@ class Line2D(Artist):
     validCap = ('butt', 'round', 'projecting')
     validJoin =   ('miter', 'round', 'bevel')
 
+    def __str__(self):
+        if self._label != "":
+            return "Line2D(%s)"%(self._label)
+        elif len(self._x) > 3:
+            return "Line2D((%g,%g),(%g,%g),...,(%g,%g))"\
+                %(self._x[0],self._y[0],self._x[0],self._y[0],self._x[-1],self._y[-1])
+        else:
+            return "Line2D(%s)"\
+                %(",".join(["(%g,%g)"%(x,y) for x,y in zip(self._x,self._y)]))
+
     def __init__(self, xdata, ydata,
                  linewidth       = None, # all Nones default to rc
                  linestyle       = None,
@@ -131,10 +184,11 @@ class Line2D(Artist):
                  markeredgecolor = None,
                  markerfacecolor = None,
                  antialiased     = None,
-                 dash_capstyle = None,
-                 solid_capstyle = None,
-                 dash_joinstyle = None,
+                 dash_capstyle   = None,
+                 solid_capstyle  = None,
+                 dash_joinstyle  = None,
                  solid_joinstyle = None,
+                 pickradius      = 5,
                  **kwargs
                  ):
         """
@@ -151,7 +205,7 @@ class Line2D(Artist):
           dash_capstyle: ['butt' | 'round' | 'projecting']
           dash_joinstyle: ['miter' | 'round' | 'bevel']
           dashes: sequence of on/off ink in points
-          data: (array xdata, array ydata)
+          data: (npy.array xdata, npy.array ydata)
           figure: a matplotlib.figure.Figure instance
           label: any string
           linestyle or ls: [ '-' | '--' | '-.' | ':' | 'steps' | 'None' | ' ' | '' ]
@@ -159,20 +213,21 @@ class Line2D(Artist):
           lod: [True | False]
           marker: [ '+' | ',' | '.' | '1' | '2' | '3' | '4'
           markeredgecolor or mec: any matplotlib color
-          markeredgewidth or mew: float value in points
+          markeredgewidth or mew: float value in points (default 5)
           markerfacecolor or mfc: any matplotlib color
           markersize or ms: float
+          pickradius: mouse event radius for pick items in points (default 5)
           solid_capstyle: ['butt' | 'round' |  'projecting']
           solid_joinstyle: ['miter' | 'round' | 'bevel']
           transform: a matplotlib.transform transformation instance
           visible: [True | False]
-          xdata: array
-          ydata: array
+          xdata: npy.array
+          ydata: npy.array
           zorder: any number
         """
         Artist.__init__(self)
 
-        #convert sequences to numeric arrays
+        #convert sequences to numpy arrays
         if not iterable(xdata):
             raise RuntimeError('xdata must be a sequence')
         if not iterable(ydata):
@@ -219,19 +274,33 @@ class Line2D(Artist):
 
         self.verticalOffset = None
 
-        self.set_data(xdata, ydata)
+        # update kwargs before updating data to give the caller a
+        # chance to init axes (and hence unit support)
+        self.update(kwargs)
+        self.pickradius = pickradius
+        if is_numlike(self._picker):
+            self.pickradius = self._picker
 
+        self.set_data(xdata, ydata)
         self._logcache = None
 
-        if len(kwargs): setp(self, **kwargs)
+        # TODO: do we really need 'newstyle'
+        self._newstyle = False
 
-    def pick(self, mouseevent):
+    def contains(self, mouseevent):
+        """Test whether the mouse event occurred on the line.  The pick radius determines
+        the precision of the location test (usually within five points of the value).  Use
+        get/set pickradius() to view or modify it.
+
+        Returns True if any values are within the radius along with {'ind': pointlist},
+        npy.where pointlist is the set of points within the radius.
+
+        TODO: sort returned indices by distance
         """
-        If mouseevent is over data that satisifies the picker, fire
-        off a backend_bases.PickEvent with the additional attribute"ind"
-        which is a sequence of indices into the data that meet the criteria
-        """
-        if not self.pickable(): return 
+        if callable(self._contains): return self._contains(self,mouseevent)
+
+        if not is_numlike(self.pickradius):
+            raise ValueError,"pick radius should be a distance"
 
         if self._newstyle:
             # transform in backend
@@ -239,29 +308,51 @@ class Line2D(Artist):
             y = self._y
         else:
             x, y = self._get_plottable()
+        if len(x)==0: return False,{}
 
-        picker = self.get_picker()
-        if is_numlike(picker):
-            eps = picker # epsilon tolerance in points
-            if len(x)==0: return
-            xt, yt = self.get_transform().numerix_x_y(x, y)
+        xt, yt = self.get_transform().numerix_x_y(x, y)
 
-            d = sqrt((xt-mouseevent.x)**2. + (yt-mouseevent.y)**2.)
-            pixels = self.figure.dpi.get()/72. * eps
-            ind = nonzero(less_equal(d, pixels))
-            if 0:
-                print 'xt', xt, mouseevent.x
-                print 'yt', yt, mouseevent.y
-                print 'd', (xt-mouseevent.x)**2., (yt-mouseevent.y)**2.
-                print d, pixels, ind
-            if len(ind):
-                self.figure.canvas.pick_event(mouseevent, self, ind=ind)
-        elif callable(picker):
-            hit, props = picker(self, mouseevent)
-            if hit:
-                self.figure.canvas.pick_event(mouseevent, self, **props)
-                
-            
+        if self.figure == None:
+            print str(self),' has no figure set'
+            pixels = self.pickradius
+        else:
+            pixels = self.figure.dpi.get()/72. * self.pickradius
+
+        if self._linestyle == 'None':
+            # If no line, return the nearby point(s)
+            d = npy.sqrt((xt-mouseevent.x)**2 + (yt-mouseevent.y)**2)
+            ind, = npy.nonzero(npy.less_equal(d, pixels))
+        else:
+            # If line, return the nearby segment(s)
+            ind = segment_hits(mouseevent.x,mouseevent.y,xt,yt,pixels)
+        if 0:
+            print 'xt', xt, mouseevent.x
+            print 'yt', yt, mouseevent.y
+            print 'd', (xt-mouseevent.x)**2., (yt-mouseevent.y)**2.
+            print d, pixels, ind
+        return len(ind)>0,dict(ind=ind)
+
+    def get_pickradius(self):
+        'return the pick radius used for containment tests'
+        return self.pickradius
+
+    def set_pickradius(self,d):
+        """Sets the pick radius used for containment tests
+
+        Accepts: float distance in points.
+        """
+        self.pickradius = d
+
+    def set_picker(self,p):
+        """Sets the event picker details for the line.
+
+        Accepts: float distance in points or callable pick function fn(artist,event)
+        """
+        if callable(p):
+            self._contains = p
+        else:
+            self.pickradius = p
+        self._picker = p
 
     def get_window_extent(self, renderer):
         self._newstyle = hasattr(renderer, 'draw_markers')
@@ -290,11 +381,18 @@ class Line2D(Artist):
         return lbwh_to_bbox( left, bottom, width, height)
 
 
+    def set_axes(self, ax):
+        Artist.set_axes(self, ax)
+        if ax.xaxis is not None:
+            self._xcid = ax.xaxis.callbacks.connect('units', self.recache)
+        if ax.yaxis is not None:
+            self._ycid = ax.yaxis.callbacks.connect('units', self.recache)
+
     def set_data(self, *args):
         """
         Set the x and y data
 
-        ACCEPTS: (array xdata, array ydata)
+        ACCEPTS: (npy.array xdata, npy.array ydata)
         """
 
         if len(args)==1:
@@ -302,15 +400,22 @@ class Line2D(Artist):
         else:
             x, y = args
 
-        self._x_orig = x
-        self._y_orig = y
+        self._xorig = x
+        self._yorig = y
+        self.recache()
+
+    def recache(self):
+        #if self.axes is None: print 'recache no axes'
+        #else: print 'recache units', self.axes.xaxis.units, self.axes.yaxis.units
+        x = ma.asarray(self.convert_xunits(self._xorig), float)
+        y = ma.asarray(self.convert_yunits(self._yorig), float)
 
         x = ma.ravel(x)
         y = ma.ravel(y)
         if len(x)==1 and len(y)>1:
-            x = x * ones(y.shape, Float)
+            x = x * npy.ones(y.shape, float)
         if len(y)==1 and len(x)>1:
-            y = y * ones(x.shape, Float)
+            y = y * npy.ones(x.shape, float)
 
         if len(x) != len(y):
             raise RuntimeError('xdata and ydata must be the same length')
@@ -325,8 +430,8 @@ class Line2D(Artist):
         else:
             self._segments = None
 
-        self._x = asarray(x, Float)
-        self._y = asarray(y, Float)
+        self._x = npy.asarray(x, float)
+        self._y = npy.asarray(y, float)
 
         self._logcache = None
 
@@ -335,7 +440,7 @@ class Line2D(Artist):
     def _is_sorted(self, x):
         "return true if x is sorted"
         if len(x)<2: return 1
-        return alltrue(x[1:]-x[0:-1]>=0)
+        return npy.alltrue(x[1:]-x[0:-1]>=0)
 
     def _get_plottable(self):
         # If log scale is set, only pos data will be returned
@@ -359,15 +464,15 @@ class Line2D(Artist):
         Nx = len(x)
         Ny = len(y)
 
-        if logx: indx = greater(x, 0)
-        else:    indx = ones(len(x))
+        if logx: indx = npy.greater(x, 0)
+        else:    indx = npy.ones(len(x))
 
-        if logy: indy = greater(y, 0)
-        else:    indy = ones(len(y))
+        if logy: indy = npy.greater(y, 0)
+        else:    indy = npy.ones(len(y))
 
-        ind = nonzero(logical_and(indx, indy))
-        x = take(x, ind)
-        y = take(y, ind)
+        ind, = npy.nonzero(npy.logical_and(indx, indy))
+        x = npy.take(x, ind)
+        y = npy.take(y, ind)
 
         self._logcache = logx, logy, x, y
         return x, y
@@ -379,13 +484,12 @@ class Line2D(Artist):
         if not self._visible: return
         self._newstyle = hasattr(renderer, 'draw_markers')
         gc = renderer.new_gc()
+        self._set_gc_clip(gc)
+
         gc.set_foreground(self._color)
         gc.set_antialiased(self._antialiased)
         gc.set_linewidth(self._linewidth)
         gc.set_alpha(self._alpha)
-        if self.get_clip_on():
-            gc.set_clip_rectangle(self.clipbox.get_bounds())
-
         if self.is_dashed():
             cap = self._dashcapstyle
             join = self._dashjoinstyle
@@ -412,23 +516,18 @@ class Line2D(Artist):
         if self._segments is not None:
             for ii in self._segments:
                 lineFunc(renderer, gc, xt[ii[0]:ii[1]], yt[ii[0]:ii[1]])
+
         else:
             lineFunc(renderer, gc, xt, yt)
 
 
         if self._marker is not None:
+
             gc = renderer.new_gc()
-            if (is_string_like(self._markeredgecolor) and
-                self._markeredgecolor == 'auto'):
-                if self._marker in self.filled_markers:
-                    gc.set_foreground('k')
-                else:
-                    gc.set_foreground(self._color)
-            else:
-                gc.set_foreground(self._markeredgecolor)
+            self._set_gc_clip(gc)
+            gc.set_foreground(self.get_markeredgecolor())
             gc.set_linewidth(self._markeredgewidth)
-            if self.get_clip_on():
-                gc.set_clip_rectangle(self.clipbox.get_bounds())
+            gc.set_alpha(self._alpha)
             funcname = self._markers.get(self._marker, '_draw_nothing')
             markerFunc = getattr(self, funcname)
             markerFunc(renderer, gc, xt, yt)
@@ -441,20 +540,52 @@ class Line2D(Artist):
 
     def get_linewidth(self): return self._linewidth
     def get_marker(self): return self._marker
-    def get_markeredgecolor(self): return self._markeredgecolor
+
+    def get_markeredgecolor(self):
+        if (is_string_like(self._markeredgecolor) and
+            self._markeredgecolor == 'auto'):
+            if self._marker in self.filled_markers:
+                return 'k'
+            else:
+                return self._color
+        else:
+            return self._markeredgecolor
+
+
+        return self._markeredgecolor
     def get_markeredgewidth(self): return self._markeredgewidth
-    def get_markerfacecolor(self): return self._markerfacecolor
+
+    def get_markerfacecolor(self):
+        if (self._markerfacecolor is None or
+            (is_string_like(self._markerfacecolor) and
+             self._markerfacecolor.lower()=='none') ):
+            return self._markerfacecolor
+        elif (is_string_like(self._markerfacecolor) and
+              self._markerfacecolor.lower() == 'auto'):
+            return self._color
+        else:
+            return self._markerfacecolor
+
+
     def get_markersize(self): return self._markersize
-    def get_xdata(self, valid_only = False):
-        if valid_only:
-            return self._x
-        return self._x_orig
-    def get_ydata(self, valid_only = False):
-        if valid_only:
-            return self._y
-        return self._y_orig
 
+    def get_xdata(self, orig=True):
+        """
+        return the xdata; if orig is true return the original data,
+        else the processed data
+        """
+        if orig:
+            return self._xorig
+        return self._x
 
+    def get_ydata(self, orig=True):
+        """
+        return the ydata; if orig is true return the original data,
+        else the processed data
+        """
+        if orig:
+            return self._yorig
+        return self._y
 
     def set_antialiased(self, b):
         """
@@ -547,9 +678,9 @@ class Line2D(Artist):
 
     def set_xdata(self, x):
         """
-        Set the data array for x
+        Set the data npy.array for x
 
-        ACCEPTS: array
+        ACCEPTS: npy.array
         """
         try: del self._xsorted
         except AttributeError: pass
@@ -558,9 +689,9 @@ class Line2D(Artist):
 
     def set_ydata(self, y):
         """
-        Set the data array for y
+        Set the data npy.array for y
 
-        ACCEPTS: array
+        ACCEPTS: npy.array
         """
 
         self.set_data(self.get_xdata(), y)
@@ -586,9 +717,9 @@ class Line2D(Artist):
     def _draw_steps(self, renderer, gc, xt, yt):
         siz=len(xt)
         if siz<2: return
-        xt2=ones((2*siz,), typecode(xt))
+        xt2=npy.ones((2*siz,), xt.dtype)
         xt2[0:-1:2], xt2[1:-1:2], xt2[-1]=xt, xt[1:], xt[-1]
-        yt2=ones((2*siz,), typecode(yt))
+        yt2=npy.ones((2*siz,), yt.dtype)
         yt2[0:-1:2], yt2[1::2]=yt, yt
         gc.set_linestyle('solid')
 
@@ -673,9 +804,9 @@ class Line2D(Artist):
         if self._newstyle:
             N = 50.0
             r = w/2.
-            rads = (2*math.pi/N)*arange(N)
-            xs = r*cos(rads)
-            ys = r*sin(rads)
+            rads = (2*math.pi/N)*npy.arange(N)
+            xs = r*npy.cos(rads)
+            ys = r*npy.sin(rads)
             # todo: use curve3!
             path = agg.path_storage()
             path.move_to(xs[0], ys[0])
@@ -685,7 +816,7 @@ class Line2D(Artist):
             path.end_poly()
             renderer.draw_markers(gc, path, rgbFace, xt, yt, self.get_transform())
         else:
-            for (x,y) in zip(xt, yt):
+            for (x,y) in zip(xt,yt):
                 renderer.draw_arc(gc, rgbFace,
                                   x, y, w, w, 0.0, 360.0, 0.0)
 
@@ -1076,6 +1207,62 @@ class Line2D(Artist):
                 renderer.draw_line(gc, x, y, x-offset2, y+offset1)
                 renderer.draw_line(gc, x, y, x-offset2, y-offset1)
 
+    def _draw_caretdown(self, renderer, gc, xt, yt):
+        offset = 0.5*renderer.points_to_pixels(self._markersize)
+        offset1 = 1.5*offset
+        if self._newstyle:
+            path = agg.path_storage()
+            path.move_to(-offset, offset1)
+            path.line_to(0, 0)
+            path.line_to(+offset, offset1)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
+        else:
+            for (x,y) in zip(xt, yt):
+                renderer.draw_line(gc, x-offset, y+offset1, x, y)
+                renderer.draw_line(gc, x, y, x+offset, y+offset1)
+
+    def _draw_caretup(self, renderer, gc, xt, yt):
+        offset = 0.5*renderer.points_to_pixels(self._markersize)
+        offset1 = 1.5*offset
+        if self._newstyle:
+            path = agg.path_storage()
+            path.move_to(-offset, -offset1)
+            path.line_to(0, 0)
+            path.line_to(+offset, -offset1)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
+        else:
+            for (x,y) in zip(xt, yt):
+                renderer.draw_line(gc, x-offset, y-offset1, x, y)
+                renderer.draw_line(gc, x, y, x+offset, y-offset1)
+
+    def _draw_caretleft(self, renderer, gc, xt, yt):
+        offset = 0.5*renderer.points_to_pixels(self._markersize)
+        offset1 = 1.5*offset
+        if self._newstyle:
+            path = agg.path_storage()
+            path.move_to(offset1, -offset)
+            path.line_to(0, 0)
+            path.line_to(offset1, offset)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
+        else:
+            for (x,y) in zip(xt, yt):
+                renderer.draw_line(gc, x+offset1, y-offset, x, y)
+                renderer.draw_line(gc, x, y, x+offset1, y+offset)
+
+    def _draw_caretright(self, renderer, gc, xt, yt):
+        offset = 0.5*renderer.points_to_pixels(self._markersize)
+        offset1 = 1.5*offset
+        if self._newstyle:
+            path = agg.path_storage()
+            path.move_to(-offset1, -offset)
+            path.line_to(0, 0)
+            path.line_to(-offset1, offset)
+            renderer.draw_markers(gc, path, None, xt, yt, self.get_transform())
+        else:
+            for (x,y) in zip(xt, yt):
+                renderer.draw_line(gc, x-offset1, y-offset, x, y)
+                renderer.draw_line(gc, x, y, x-offset1, y+offset)
+
     def _draw_x(self, renderer, gc, xt, yt):
         offset = 0.5*renderer.points_to_pixels(self._markersize)
 
@@ -1112,15 +1299,11 @@ class Line2D(Artist):
 
 
     def _get_rgb_face(self):
-        if (self._markerfacecolor is None or
-            (is_string_like(self._markerfacecolor) and
-             self._markerfacecolor.lower()=='none') ):
+        facecolor = self.get_markerfacecolor()
+        if is_string_like(facecolor) and facecolor.lower()=='none':
             rgbFace = None
-        elif (is_string_like(self._markerfacecolor) and
-              self._markerfacecolor.lower() == 'auto'):
-            rgbFace = colorConverter.to_rgb(self._color)
         else:
-            rgbFace = colorConverter.to_rgb(self._markerfacecolor)
+            rgbFace = colorConverter.to_rgb(facecolor)
         return rgbFace
 
     # some aliases....
