@@ -16,7 +16,18 @@ OVERVIEW
 
   s = r'$\mathcal{R}\prod_{i=\alpha\mathcal{B}}^\infty a_i\sin(2 \pi f x_i)$'
 
-  The fonts \cal, \rm, \it, and \tt are allowed.
+  Different fonts may be selected:
+    \mathcal      Calligraphic fonts
+    \mathrm       Roman (upright) font
+    \mathit       Italic font
+    \mathtt       Typewriter (monospaced) font, similar to Courier
+
+  Additionally, if using the STIX fonts:
+    \mathbb       Blackboard (double-struck) font
+    \mathcircled  Circled characters
+    \mathfrak     Fraktur (Gothic-style) font
+    \mathscr      Script (cursive) font
+    \mathsf       Sans-serif font
 
   The following accents are provided: \hat, \breve, \grave, \bar,
   \acute, \tilde, \vec, \dot, \ddot.  All of them have the same
@@ -70,47 +81,84 @@ USAGE:
 
   Allowed TeX symbols:
 
-  [MGDTODO: This list is no longer exhaustive and needs to be updated]
-
-  \/ \Delta \Downarrow \Gamma \Im \LEFTangle \LEFTbrace \LEFTbracket
-  \LEFTparen \Lambda \Leftarrow \Leftbrace \Leftbracket \Leftparen
-  \Leftrightarrow \Omega \P \Phi \Pi \Psi \RIGHTangle \RIGHTbrace
-  \RIGHTbracket \RIGHTparen \Re \Rightarrow \Rightbrace \Rightbracket
-  \Rightparen \S \SQRT \Sigma \Sqrt \Theta \Uparrow \Updownarrow
-  \Upsilon \Vert \Xi \aleph \alpha \approx \angstrom \ast \asymp
-  \backslash \beta \bigcap \bigcirc \bigcup \bigodot \bigoplus
-  \bigotimes \bigtriangledown \bigtriangleup \biguplus \bigvee
-  \bigwedge \bot \bullet \cap \cdot \chi \circ \clubsuit \coprod \cup
-  \dag \dashv \ddag \delta \diamond \diamondsuit \div \downarrow \ell
-  \emptyset \epsilon \equiv \eta \exists \flat \forall \frown \gamma
-  \geq \gg \heartsuit \hspace \imath \in \infty \int \iota \jmath
-  \kappa \lambda \langle \lbrace \lceil \leftangle \leftarrow
-  \leftbrace \leftbracket \leftharpoondown \leftharpoonup \leftparen
-  \leftrightarrow \leq \lfloor \ll \mid \mp \mu \nabla \natural
-  \nearrow \neg \ni \nu \nwarrow \odot \oint \omega \ominus \oplus
-  \oslash \otimes \phi \pi \pm \prec \preceq \prime \prod \propto \psi
-  \rangle \rbrace \rceil \rfloor \rho \rightangle \rightarrow
-  \rightbrace \rightbracket \rightharpoondown \rightharpoonup
-  \rightparen \searrow \sharp \sigma \sim \simeq \slash \smile
-  \spadesuit \sqcap \sqcup \sqrt \sqsubseteq \sqsupseteq \subset
-  \subseteq \succ \succeq \sum \supset \supseteq \swarrow \tau \theta
-  \times \top \triangleleft \triangleright \uparrow \updownarrow
-  \uplus \upsilon \varepsilon \varphi \varphi \varrho \varsigma
-  \vartheta \vdash \vee \vert \wedge \wp \wr \xi \zeta
-
+  $ \% \AA \AE \BbbC \BbbN \BbbP \BbbQ \BbbR \BbbZ \Bumpeq \Cap \Colon
+  \Cup \Delta \Doteq \Downarrow \Equiv \Finv \Gamma \H \Im \L \Lambda
+  \Ldsh \Leftarrow \Leftrightarrow \Lleftarrow \Lsh \Nearrow \Nwarrow
+  \O \OE \Omega \P \Phi \Pi \Psi \Rdsh \Re \Rightarrow \Rrightarrow
+  \Rsh \S \Searrow \Sigma \Subset \Supset \Swarrow \Theta \Uparrow
+  \Updownarrow \Upsilon \Vdash \Vert \Vvdash \Xi \_ \__sqrt__ \ac
+  \acute \acwopencirclearrow \adots \ae \aleph \alpha \angle \approx
+  \approxeq \approxident \arceq \ast \asymp \backcong \backprime
+  \backsim \backsimeq \backslash \bar \barleftarrow \barwedge \because
+  \beta \beth \between \bigcap \bigcirc \bigcup \bigodot \bigoplus
+  \bigotimes \bigstar \bigtriangledown \bigtriangleup \biguplus
+  \bigvee \bigwedge \blacksquare \blacktriangle \blacktriangledown
+  \blacktriangleleft \blacktriangleright \bot \bowtie \boxbar \boxdot
+  \boxminus \boxplus \boxtimes \breve \bullet \bumpeq \c \candra \cap
+  \carriagereturn \cdot \cdotp \cdots \check \checkmark \chi \circ
+  \circeq \circledR \circledS \circledast \circledcirc \circleddash
+  \circumflexaccent \clubsuit \clubsuitopen \colon \coloneq
+  \combiningacuteaccent \combiningbreve \combiningdiaeresis
+  \combiningdotabove \combininggraveaccent \combiningoverline
+  \combiningrightarrowabove \combiningtilde \complement \cong \coprod
+  \copyright \cup \cupdot \curlyeqprec \curlyeqsucc \curlyvee
+  \curlywedge \curvearrowleft \curvearrowright \cwopencirclearrow \d
+  \dag \daleth \danger \dashv \ddag \ddddot \dddot \ddot \ddots
+  \degree \delta \diamond \diamondsuit \digamma \div \divideontimes
+  \dot \doteq \dotminus \dotplus \dots \doublebarwedge ? \downarrow
+  \downdownarrows \downharpoonleft \downharpoonright \downzigzagarrow
+  \ell \emdash \emptyset \endash \enspace \epsilon \eqcirc \eqcolon
+  \eqdef \eqgtr \eqless \eqsim \equiv \eta \eth \exists \fallingdotseq
+  \flat \forall \frakC \frakZ \frown \gamma \geq \geqq \gg \ggg \gimel
+  \gneqq \gnsim \grave \greater \gtrdot \gtreqless \gtrless \gtrsim
+  \hat \heartsuit \hookleftarrow \hookrightarrow \i \iiint \iint
+  \imageof \imath \in \infty \int \intercal \invnot \iota \jmath \k
+  \kappa \kernelcontraction \l \lambda \lambdabar \lasp \lbrace
+  \lbrack \lceil \leftangle \leftarrow \leftarrowtail \leftbrace
+  \leftharpoonaccent \leftharpoondown \leftharpoonup \leftleftarrows
+  \leftparen \leftrightarrow \leftrightarrows \leftrightharpoons
+  \leftthreetimes \leq \leqq \less \lessdot \lesseqgtr \lessgtr
+  \lesssim \lfloor \ll \llcorner \lll \lneqq \lnsim \looparrowleft
+  \looparrowright \lq \lrcorner \ltimes \maltese \mapsdown \mapsfrom
+  \mapsto \mapsup \measeq \measuredangle \mho \mid \minus \models \mp
+  \mu \multimap \nLeftarrow \nLeftrightarrow \nRightarrow \nVDash
+  \nVdash \nabla \napprox \natural \ncong \ne \nearrow \neg \nequiv
+  \nexists \ngeq \ngtr \ni \nleftarrow \nleftrightarrow \nleq \nless
+  \nmid \not \notin \nparallel \nprec \nrightarrow \nsim \nsime
+  \nsubset \nsubseteq \nsucc \nsupset \nsupseteq \ntriangleleft
+  \ntrianglelefteq \ntriangleright \ntrianglerighteq \nu \nvDash
+  \nvdash \nwarrow \o \obar \ocirc \odot \oe \oiiint \oiint \oint
+  \omega \ominus \oplus \origof \oslash \otimes \overarc
+  \overleftarrow \overleftrightarrow \parallel \partial \phi \pi
+  \pitchfork \pm \prec \preccurlyeq \preceq \precnsim \precsim \prime
+  \prod \propto \prurel \psi \quad \questeq \rasp \rbrace \rbrack
+  \rceil \rfloor \rho \rightangle \rightarrow \rightarrowbar
+  \rightarrowtail \rightbrace \rightharpoonaccent \rightharpoondown
+  \rightharpoonup \rightleftarrows \rightleftharpoons \rightparen
+  \rightrightarrows \rightthreetimes \rightzigzagarrow \risingdotseq
+  \rq \rtimes \scrB \scrE \scrF \scrH \scrI \scrL \scrM \scrR \scre
+  \scrg \scro \scurel \searrow \sharp \sigma \sim \simeq \slash
+  \smallsetminus \smile \solbar \spadesuit \spadesuitopen
+  \sphericalangle \sqcap \sqcup \sqsubset \sqsubseteq \sqsupset
+  \sqsupseteq \ss \star \stareq \sterling \subset \subseteq \subsetneq
+  \succ \succcurlyeq \succeq \succnsim \succsim \sum \supset \supseteq
+  \supsetneq \swarrow \t \tau \textasciiacute \textasciicircum
+  \textasciigrave \textasciitilde \textexclamdown \textquestiondown
+  \textquotedblleft \textquotedblright \therefore \theta \thickspace
+  \thinspace \tilde \times \to \top \triangledown \triangleleft
+  \trianglelefteq \triangleq \triangleright \trianglerighteq
+  \turnednot \twoheaddownarrow \twoheadleftarrow \twoheadrightarrow
+  \twoheaduparrow \ulcorner \underbar \uparrow \updownarrow
+  \updownarrowbar \updownarrows \upharpoonleft \upharpoonright \uplus
+  \upsilon \upuparrows \urcorner \vDash \varepsilon \varkappa
+  \varnothing \varphi \varpi \varrho \varsigma \vartheta \vartriangle
+  \vartriangleleft \vartriangleright \vdash \vdots \vec \vee \veebar
+  \veeeq \vert \wedge \wedgeq \widehat \widetilde \wp \wr \xi \yen
+  \zeta \{ \| \}
 
 BACKENDS
 
-  mathtext currently works with GTK, Agg, GTKAgg, TkAgg and WxAgg and
-  PS, though only horizontal and vertical rotations are supported in
-  *Agg
-
-  mathtext now embeds the TrueType computer modern fonts into the PS
-  file, so what you see on the screen should be what you get on paper.
-
-  Backends which don't support mathtext will just render the TeX
-  string as a literal.  Stay tuned.
-
+  mathtext currently works with all backends.
 
 KNOWN ISSUES:
 
@@ -541,10 +589,7 @@ class TruetypeFonts(Fonts):
 
         cached_font = self._fonts.get(basename)
         if cached_font is None:
-            try:
-                font = FT2Font(basename)
-            except RuntimeError:
-                return None
+            font = FT2Font(basename)
             cached_font = self.CachedFont(font)
             self._fonts[basename] = cached_font
             self._fonts[font.postscript_name] = cached_font
@@ -652,13 +697,20 @@ class BakomaFonts(TruetypeFonts):
         if fontname in self.fontmap and latex_to_bakoma.has_key(sym):
             basename, num = latex_to_bakoma[sym]
             slanted = (basename == "cmmi10") or sym in self._slanted_symbols
-            cached_font = self._get_font(basename)
-            symbol_name = cached_font.font.get_glyph_name(num)
-            num = cached_font.glyphmap[num]
+            try:
+                cached_font = self._get_font(basename)
+            except RuntimeError:
+                pass
+            else:
+                symbol_name = cached_font.font.get_glyph_name(num)
+                num = cached_font.glyphmap[num]
         elif len(sym) == 1:
             slanted = (fontname == "it")
-            cached_font = self._get_font(fontname)
-            if cached_font is not None:
+            try:
+                cached_font = self._get_font(fontname)
+            except RuntimeError:
+                pass
+            else:
                 num = ord(sym)
                 gid = cached_font.charmap.get(num)
                 if gid is not None:
@@ -795,9 +847,12 @@ class UnicodeFonts(TruetypeFonts):
                     new_fontname = 'rm'
 
             slanted = (new_fontname == 'it') or sym in self._slanted_symbols
-            cached_font = self._get_font(new_fontname)
             found_symbol = False
-            if cached_font is not None:
+            try:
+                cached_font = self._get_font(new_fontname)
+            except RuntimeError:
+                pass
+            else:
                 try:
                     glyphindex = cached_font.charmap[uniindex]
                     found_symbol = True
@@ -2003,7 +2058,7 @@ class Parser(object):
                      ).setParseAction(self.customspace).setName('customspace')
 
         unicode_range = u"\U00000080-\U0001ffff"
-        symbol       =(Regex(UR"([a-zA-Z0-9 +\-*/<>=:,.;!'@()%s])|(\\[%%${}\[\]_])" % unicode_range)
+        symbol       =(Regex(UR"([a-zA-Z0-9 +\-*/<>=:,.;!'@()|%s])|(\\[%%${}\[\]_|])" % unicode_range)
                      | Combine(
                          bslash
                        + oneOf(tex2uni.keys())
