@@ -48,13 +48,13 @@ commands with the same names.
 
    * rec2csv          : store record array in CSV file
    * rec2excel        : store record array in excel worksheet - required pyExcelerator
-   * rec2gtk          : put record array in GTK treeview - requires gtk
+
    * csv2rec          : import record array from CSV file with type inspection
    * rec_append_field : add a field/array to record array
    * rec_drop_fields  : drop fields from record array
    * rec_join         : join two record arrays on sequence of fields
 
-For the rec viewer clases (rec2csv, rec2excel and rec2gtk), there are
+For the rec viewer clases (rec2csv, rec2excel), there are
 a bunch of Format objects you can pass into the functions that will do
 things like color negative values red, set percent formatting and
 scaling, etc.
@@ -1978,7 +1978,7 @@ def rec_join(key, r1, r2):
     join record arrays r1 and r2 on key; key is a tuple of field
     names.  if r1 and r2 have equal values on all the keys in the key
     tuple, then their fields will be merged into a new record array
-    containing the union of the fields of r1 and r2
+    containing the intersection of the fields of r1 and r2
     """
 
     for name in key:
@@ -2045,7 +2045,7 @@ def rec_join(key, r1, r2):
     return newrec.view(npy.recarray)
 
 
-def csv2rec(fname, comments='#', skiprows=0, checkrows=5, delimiter=',',
+def csv2rec(fname, comments='#', skiprows=0, checkrows=0, delimiter=',',
             converterd=None, names=None, missing=None):
     """
     Load data from comma/space/tab delimited file in fname into a
@@ -2075,7 +2075,7 @@ def csv2rec(fname, comments='#', skiprows=0, checkrows=5, delimiter=',',
     names, if not None, is a list of header names.  In this case, no
     header will be read from the file
 
-    if no rows are found, None is returned See examples/loadrec.py
+    if no rows are found, None is returned -- see examples/loadrec.py
     """
 
     if converterd is None:
@@ -2129,6 +2129,7 @@ def csv2rec(fname, comments='#', skiprows=0, checkrows=5, delimiter=',',
 
     process_skiprows(reader)
 
+    dateparser = dateutil.parser.parse
 
     def myfloat(x):
         if x==missing:
@@ -2136,9 +2137,18 @@ def csv2rec(fname, comments='#', skiprows=0, checkrows=5, delimiter=',',
         else:
             return float(x)
 
+    def mydate(x):
+        # try and return a date object
+        d = dateparser(x)
+
+        if d.hour>0 or d.minute>0 or d.second>0:
+            raise ValueError('not a date')
+        return d.date()
+
+
     def get_func(item, func):
         # promote functions in this order
-        funcmap = {int:myfloat, myfloat:dateutil.parser.parse, dateutil.parser.parse:str}
+        funcmap = {int:myfloat, myfloat:mydate, mydate:dateparser, dateparser:str}
         try: func(item)
         except:
             if func==str:
@@ -2238,8 +2248,12 @@ class FormatObj:
 
 class FormatString(FormatObj):
     def tostr(self, x):
-        return '"%s"'%self.toval(x)
+        val = repr(x)
+        return val[1:-1]
 
+#class FormatString(FormatObj):
+#    def tostr(self, x):
+#        return '"%r"'%self.toval(x)
 
 class FormatFormatStr(FormatObj):
     def __init__(self, fmt):
@@ -2297,7 +2311,7 @@ defaultformatd = {
     npy.float32 : FormatFloat(),
     npy.float64 : FormatFloat(),
     npy.object_ : FormatObj(),
-    npy.string_ : FormatObj(),
+    npy.string_ : FormatString(),
     }
 
 def get_formatd(r, formatd=None):
@@ -2317,7 +2331,7 @@ def csvformat_factory(format):
     format = copy.deepcopy(format)
     if isinstance(format, FormatFloat):
         format.scale = 1. # override scaling for storage
-        format.fmt = '%g' # maximal precision
+        format.fmt = '%r'
     return format
 
 def rec2csv(r, fname, delimiter=',', formatd=None):
@@ -2335,373 +2349,14 @@ def rec2csv(r, fname, delimiter=',', formatd=None):
     for i, name in enumerate(r.dtype.names):
         funcs.append(csvformat_factory(formatd[name]).tostr)
 
-    fh = cbook.to_filehandle(fname, 'w')
+    fh, opened = cbook.to_filehandle(fname, 'w', return_opened=True)
     writer = csv.writer(fh, delimiter=delimiter)
     header = r.dtype.names
     writer.writerow(header)
     for row in r:
         writer.writerow([func(val) for func, val in zip(funcs, row)])
-    fh.close()
+    if opened:
+        fh.close()
 
-# if pyExcelerator is installed, provide an excel view
-try:
-    import pyExcelerator as excel
-except ImportError:
-    pass
-else:
-
-    def xlformat_factory(format):
-        """
-        copy the format, perform any overrides, and attach an xlstyle instance
-        copied format is returned
-        """
-        format = copy.deepcopy(format)
-
-
-
-        xlstyle = excel.XFStyle()
-        if isinstance(format, FormatFloat):
-            zeros = ''.join(['0']*format.precision)
-            xlstyle.num_format_str = '#,##0.%s;[RED]-#,##0.%s'%(zeros, zeros)
-        elif isinstance(format, FormatInt):
-            xlstyle.num_format_str = '#,##;[RED]-#,##'
-        elif isinstance(format, FormatPercent):
-            zeros = ''.join(['0']*format.precision)
-            xlstyle.num_format_str = '0.%s%;[RED]-0.%s%'%(zeros, zeros)
-            format.scale = 1.
-        else:
-            xlstyle = None
-
-        format.xlstyle = xlstyle
-
-        return format
-
-    def rec2excel(r, ws, formatd=None, rownum=0):
-        """
-        save record array r to excel pyExcelerator worksheet ws
-        starting at rownum.  if ws is string like, assume it is a
-        filename and save to it
-
-        formatd is a dictionary mapping dtype name -> FormatXL instances
-
-        The next rownum after writing is returned
-        """
-
-        autosave = False
-        if cbook.is_string_like(ws):
-            filename = ws
-            wb = excel.Workbook()
-            ws = wb.add_sheet('worksheet')
-            autosave = True
-
-
-        if formatd is None:
-            formatd = dict()
-
-        formats = []
-        for i, name in enumerate(r.dtype.names):
-            dt = r.dtype[name]
-            format = formatd.get(name)
-            if format is None:
-                format = defaultformatd.get(dt.type, FormatObj())
-
-            format = xlformat_factory(format)
-            ws.write(rownum, i, name)
-            formats.append(format)
-
-        rownum+=1
-
-
-        ind = npy.arange(len(r.dtype.names))
-        for row in r:
-            for i in ind:
-                val = row[i]
-                format = formats[i]
-                val = format.toval(val)
-                if format.xlstyle is None:
-                    ws.write(rownum, i, val)
-                else:
-                    if safe_isnan(val):
-                        ws.write(rownum, i, 'NaN')
-                    else:
-                        ws.write(rownum, i, val, format.xlstyle)
-            rownum += 1
-
-        if autosave:
-            wb.save(filename)
-        return rownum
-
-
-
-
-# if gtk is installed, provide a gtk view
-try:
-    import gtk, gobject
-except ImportError:
-    pass
-except RuntimeError:
-    pass
-
-
-
-    def gtkformat_factory(format, colnum):
-        """
-        copy the format, perform any overrides, and attach an gtk style attrs
-
-
-        xalign = 0.
-        cell = None
-
-        """
-
-        format = copy.copy(format)
-        format.xalign = 0.
-        format.cell = None
-
-        def negative_red_cell(column, cell, model, thisiter):
-            val = model.get_value(thisiter, colnum)
-            try: val = float(val)
-            except: cell.set_property('foreground', 'black')
-            else:
-                if val<0:
-                    cell.set_property('foreground', 'red')
-                else:
-                    cell.set_property('foreground', 'black')
-
-
-        if isinstance(format, FormatFloat) or isinstance(format, FormatInt):
-            format.cell = negative_red_cell
-            format.xalign = 1.
-        elif isinstance(format, FormatDate):
-            format.xalign = 1.
-        return format
-
-
-
-    class SortedStringsScrolledWindow(gtk.ScrolledWindow):
-        """
-        A simple treeview/liststore assuming all columns are strings.
-        Supports ascending/descending sort by clicking on column header
-        """
-
-        def __init__(self, colheaders, formatterd=None):
-            """
-            xalignd if not None, is a dict mapping col header to xalignent (default 1)
-
-            formatterd if not None, is a dict mapping col header to a ColumnFormatter
-            """
-
-
-            gtk.ScrolledWindow.__init__(self)
-            self.colheaders = colheaders
-            self.seq = None # not initialized with accts
-            self.set_shadow_type(gtk.SHADOW_ETCHED_IN)
-            self.set_policy(gtk.POLICY_AUTOMATIC,
-                            gtk.POLICY_AUTOMATIC)
-
-            types = [gobject.TYPE_STRING] * len(colheaders)
-            model = self.model = gtk.ListStore(*types)
-
-
-            treeview = gtk.TreeView(self.model)
-            treeview.show()
-            treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
-            treeview.set_rules_hint(True)
-
-
-            class Clicked:
-                def __init__(self, parent, i):
-                    self.parent = parent
-                    self.i = i
-                    self.num = 0
-
-                def __call__(self, column):
-                    ind = []
-                    dsu = []
-                    for rownum, thisiter in enumerate(self.parent.iters):
-                        val = model.get_value(thisiter, self.i)
-                        try: val = float(val.strip().rstrip('%'))
-                        except ValueError: pass
-                        if npy.isnan(val): val = npy.inf # force nan to sort uniquely
-                        dsu.append((val, rownum))
-                    dsu.sort()
-                    if not self.num%2: dsu.reverse()
-
-                    vals, otherind = zip(*dsu)
-                    ind.extend(otherind)
-
-                    self.parent.model.reorder(ind)
-                    newiters = []
-                    for i in ind:
-                        newiters.append(self.parent.iters[i])
-                    self.parent.iters = newiters[:]
-                    for i, thisiter in enumerate(self.parent.iters):
-                        key = tuple([self.parent.model.get_value(thisiter, j) for j in range(len(colheaders))])
-                        self.parent.rownumd[i] = key
-
-                    self.num+=1
-
-
-            if formatterd is None:
-                formatterd = dict()
-
-            formatterd = formatterd.copy()
-
-            for i, header in enumerate(colheaders):
-                renderer = gtk.CellRendererText()
-                if header not in formatterd:
-                    formatterd[header] = ColumnFormatter()
-                formatter = formatterd[header]
-
-                column = gtk.TreeViewColumn(header, renderer, text=i)
-                renderer.set_property('xalign', formatter.xalign)
-                column.connect('clicked', Clicked(self, i))
-                column.set_property('clickable', True)
-
-                if formatter.cell is not None:
-                    column.set_cell_data_func(renderer, formatter.cell)
-
-                treeview.append_column(column)
-
-
-
-            self.formatterd = formatterd
-            self.lastcol = column
-            self.add(treeview)
-            self.treeview = treeview
-            self.clear()
-
-        def clear(self):
-            self.iterd = dict()
-            self.iters = []        # an ordered list of iters
-            self.rownumd = dict()  # a map from rownum -> symbol
-            self.model.clear()
-            self.datad = dict()
-
-
-        def flat(self, row):
-            seq = []
-            for i,val in enumerate(row):
-                formatter = self.formatterd.get(self.colheaders[i])
-                seq.extend([i,formatter.tostr(val)])
-            return seq
-
-        def __delete_selected(self, *unused): # untested
-
-
-            keyd = dict([(thisiter, key) for key, thisiter in self.iterd.values()])
-            for row in self.get_selected():
-                key = tuple(row)
-                thisiter = self.iterd[key]
-                self.model.remove(thisiter)
-                del self.datad[key]
-                del self.iterd[key]
-                self.iters.remove(thisiter)
-
-            for i, thisiter in enumerate(self.iters):
-                self.rownumd[i] = keyd[thisiter]
-
-
-
-        def delete_row(self, row):
-            key = tuple(row)
-            thisiter = self.iterd[key]
-            self.model.remove(thisiter)
-
-
-            del self.datad[key]
-            del self.iterd[key]
-            self.rownumd[len(self.iters)] = key
-            self.iters.remove(thisiter)
-
-            for rownum, thiskey in self.rownumd.items():
-                if thiskey==key: del self.rownumd[rownum]
-
-        def add_row(self, row):
-            thisiter = self.model.append()
-            self.model.set(thisiter, *self.flat(row))
-            key = tuple(row)
-            self.datad[key] = row
-            self.iterd[key] = thisiter
-            self.rownumd[len(self.iters)] = key
-            self.iters.append(thisiter)
-
-        def update_row(self, rownum, newrow):
-            key = self.rownumd[rownum]
-            thisiter = self.iterd[key]
-            newkey = tuple(newrow)
-
-            self.rownumd[rownum] = newkey
-            del self.datad[key]
-            del self.iterd[key]
-            self.datad[newkey] = newrow
-            self.iterd[newkey] = thisiter
-
-
-            self.model.set(thisiter, *self.flat(newrow))
-
-        def get_row(self, rownum):
-            key = self.rownumd[rownum]
-            return self.datad[key]
-
-        def get_selected(self):
-            selected = []
-            def foreach(model, path, iter, selected):
-                selected.append(model.get_value(iter, 0))
-
-            self.treeview.get_selection().selected_foreach(foreach, selected)
-            return selected
-
-
-
-    def rec2gtk(r, formatd=None, rownum=0, autowin=True):
-        """
-        save record array r to excel pyExcelerator worksheet ws
-        starting at rownum.  if ws is string like, assume it is a
-        filename and save to it
-
-        formatd is a dictionary mapping dtype name -> FormatXL instances
-
-        This function creates a SortedStringsScrolledWindow (derived
-        from gtk.ScrolledWindow) and returns it.  if autowin is True,
-        a gtk.Window is created, attached to the
-        SortedStringsScrolledWindow instance, shown and returned.  If
-        autowin=False, the caller is responsible for adding the
-        SortedStringsScrolledWindow instance to a gtk widget and
-        showing it.
-        """
-
-
-
-        if formatd is None:
-            formatd = dict()
-
-        formats = []
-        for i, name in enumerate(r.dtype.names):
-            dt = r.dtype[name]
-            format = formatd.get(name)
-            if format is None:
-                format = defaultformatd.get(dt.type, FormatObj())
-            #print 'gtk fmt factory', i, name, format, type(format)
-            format = gtkformat_factory(format, i)
-            formatd[name] = format
-
-
-        colheaders = r.dtype.names
-        scroll = SortedStringsScrolledWindow(colheaders, formatd)
-
-        ind = npy.arange(len(r.dtype.names))
-        for row in r:
-            scroll.add_row(row)
-
-
-        if autowin:
-            win = gtk.Window()
-            win.set_default_size(800,600)
-            win.add(scroll)
-            win.show_all()
-            scroll.win = win
-
-        return scroll
 
 
