@@ -143,7 +143,7 @@ def validate_color(s):
 
     if len(s)==6 and s.isalnum(): # looks like hex
         return '#' + s
-    
+
     if len(s)==7 and s.startswith('#') and s[1:].isalnum():
         return s
 
@@ -165,21 +165,6 @@ validate_orientation = ValidateInStrings('orientation',[
     'landscape', 'portrait',
     ])
 
-def validate_latex_preamble(s):
-    'return a list'
-    preamble_list = validate_stringlist(s)
-    if not preamble_list == ['']:
-        verbose.report("""
-*****************************************************************
-You have the following UNSUPPORTED LaTeX preamble customizations:
-%s
-Please do not ask for support with these customizations active.
-*****************************************************************
-"""% '\n'.join(preamble_list), 'helpful')
-    return preamble_list
-
-
-
 def validate_aspect(s):
     if s in ('auto', 'equal'):
         return s
@@ -198,7 +183,7 @@ def validate_fontsize(s):
         return float(s)
     except ValueError:
         raise ValueError('not a valid font size')
-    
+
 def validate_font_properties(s):
     parse_fontconfig_pattern(s)
     return s
@@ -290,7 +275,7 @@ class ValidateInterval:
 
 # a map from key -> value, converter
 defaultParams = {
-    'backend'           : ['WXAgg', validate_backend],
+    'backend'           : ['Agg', validate_backend], # agg is certainly present
     'numerix'           : ['numpy', validate_numerix],
     'maskedarray'       : [False, validate_bool],
     'toolbar'           : ['toolbar2', validate_toolbar],
@@ -353,7 +338,7 @@ defaultParams = {
     'text.color'          : ['k', validate_color],     # black
     'text.usetex'         : [False, validate_bool],
     'text.latex.unicode'  : [False, validate_bool],
-    'text.latex.preamble' : [[''], validate_latex_preamble],
+    'text.latex.preamble' : [[''], validate_stringlist],
     'text.dvipnghack'     : [False, validate_bool],
     'text.fontstyle'      : ['normal', str],
     'text.fontangle'      : ['normal', str],
@@ -369,7 +354,7 @@ defaultParams = {
     'mathtext.sf'         : ['sans\-serif', validate_font_properties],
     'mathtext.fontset'    : ['cm', validate_fontset],
     'mathtext.fallback_to_cm' : [True, validate_bool],
-    
+
     'image.aspect'        : ['equal', validate_aspect],  # equal, auto, a number
     'image.interpolation' : ['bilinear', str],
     'image.cmap'          : ['jet', str],        # one of gray, jet, etc
