@@ -79,7 +79,8 @@ WARNING: found a TeX cache dir in the deprecated location "%s".
     dvipngVersion = get_dvipng_version()
 
     # mappable cache of
-    arrayd = {}
+    rgba_arrayd = {}
+    grey_arrayd = {}
     postscriptd = {}
     pscnt = 0
 
@@ -131,7 +132,7 @@ WARNING: found a TeX cache dir in the deprecated location "%s".
                     found_font = self.font_info[font.lower()]
                     setattr(self, font_family_attr,
                             self.font_info[font.lower()])
-                    if DEBUG: 
+                    if DEBUG:
                         print 'family: %s, font: %s, info: %s'%(font_family,
                                     font, self.font_info[font.lower()])
                     break
@@ -323,6 +324,24 @@ process the flowing file:\n%s\nHere is the full report generated by dvipng: \
                 return [int(val) for val in line.split()[1:]]
         raise RuntimeError('Could not parse %s'%psfile)
 
+    def get_grey(self, tex, fontsize=None, dpi=None):
+        key = tex, self.get_font_config(), fontsize, dpi
+        alpha = self.grey_arrayd.get(key)
+
+        if alpha is None:
+            pngfile = self.make_png(tex, fontsize, dpi)
+            X = readpng(os.path.join(self.texcache, pngfile))
+
+            if (self.dvipngVersion < '1.6') or rcParams['text.dvipnghack']:
+                # hack the alpha channel as described in comment above
+                alpha = npy.sqrt(1-X[:,:,0])
+            else:
+                alpha = X[:,:,-1]
+
+            self.grey_arrayd[key] = alpha
+        return alpha
+
+
     def get_rgba(self, tex, fontsize=None, dpi=None, rgb=(0,0,0)):
         """
         Return tex string as an rgba array
@@ -351,23 +370,16 @@ process the flowing file:\n%s\nHere is the full report generated by dvipng: \
         if not dpi: dpi = rcParams['savefig.dpi']
         r,g,b = rgb
         key = tex, self.get_font_config(), fontsize, dpi, tuple(rgb)
-        Z = self.arrayd.get(key)
+        Z = self.rgba_arrayd.get(key)
 
         if Z is None:
-            pngfile = self.make_png(tex, fontsize, dpi)
-            X = readpng(os.path.join(self.texcache, pngfile))
+            alpha = self.get_grey(tex, fontsize, dpi)
 
-            if (self.dvipngVersion < '1.6') or rcParams['text.dvipnghack']:
-                # hack the alpha channel as described in comment above
-                alpha = npy.sqrt(1-X[:,:,0])
-            else:
-                alpha = X[:,:,-1]
-
-            Z = npy.zeros(X.shape, npy.float)
+            Z = npy.zeros((X.shape[0], X.shape[1], 4), npy.float)
             Z[:,:,0] = r
             Z[:,:,1] = g
             Z[:,:,2] = b
             Z[:,:,3] = alpha
-            self.arrayd[key] = Z
+            self.rgba_arrayd[key] = Z
 
         return Z
