@@ -111,14 +111,15 @@ multiple axes can share the same locator w/o side effects!
 
 from __future__ import division
 import sys, os, re, time, math, warnings
-from mlab import linspace
+import numpy as npy
+import matplotlib as mpl
 from matplotlib import verbose, rcParams
-from numerix import absolute, arange, array, asarray, Float, floor, log, \
-     logical_and, nonzero, ones, take, zeros
-from matplotlib.numerix.mlab import amin, amax, std, mean
-from matplotlib.mlab import frange
-from cbook import strip_math
-from transforms import nonsingular, Value, Interval
+from matplotlib import cbook
+from matplotlib import transforms as mtrans
+
+
+
+
 
 class TickHelper:
 
@@ -148,8 +149,8 @@ class TickHelper:
         cases where the Intervals do not need to be updated
         automatically.
         '''
-        self.dataInterval = Interval(Value(vmin), Value(vmax))
-        self.viewInterval = Interval(Value(vmin), Value(vmax))
+        self.dataInterval = mtrans.Interval(mtrans.Value(vmin), mtrans.Value(vmax))
+        self.viewInterval = mtrans.Interval(mtrans.Value(vmin), mtrans.Value(vmax))
 
 class Formatter(TickHelper):
     """
@@ -274,7 +275,7 @@ class ScalarFormatter(Formatter):
     def __init__(self, useOffset=True, useMathText=False):
         # useOffset allows plotting small data ranges with large offsets:
         # for example: [1+1e-9,1+2e-9,1+3e-9]
-        # useMathText will render the offset an scientific notation in mathtext
+        # useMathText will render the offset and scientific notation in mathtext
         self._useOffset = useOffset
         self._usetex = rcParams['text.usetex']
         self._useMathText = useMathText
@@ -292,7 +293,9 @@ class ScalarFormatter(Formatter):
             return self.pprint_val(x)
 
     def set_scientific(self, b):
-        'True or False to turn scientific notation on or off; see also set_powerlimits()'
+        '''True or False to turn scientific notation on or off
+        see also set_powerlimits()
+        '''
         self._scientific = bool(b)
 
     def set_powerlimits(self, lims):
@@ -311,11 +314,9 @@ class ScalarFormatter(Formatter):
         'return a short formatted string representation of a number'
         return '%1.3g'%value
 
-    def format_data(self,value,sign=False,mathtext=False):
+    def format_data(self,value):
         'return a formatted string representation of a number'
-        if sign: s = '%+1.10e'% value
-        else: s = '%1.10e'% value
-        return self._formatSciNotation(s,mathtext=mathtext)
+        return self._formatSciNotation('%1.10e'% value)
 
     def get_offset(self):
         """Return scientific notation, plus offset"""
@@ -324,17 +325,19 @@ class ScalarFormatter(Formatter):
             offsetStr = ''
             sciNotStr = ''
             if self.offset:
-                if self._usetex or self._useMathText:
-                    offsetStr = self.format_data(self.offset, sign=True, mathtext=True)
-                else:
-                    offsetStr = self.format_data(self.offset, sign=True, mathtext=False)
+                offsetStr = self.format_data(self.offset)
+                if self.offset > 0: offsetStr = '+' + offsetStr
             if self.orderOfMagnitude:
                 if self._usetex or self._useMathText:
-                    sciNotStr = r'{\times}'+self.format_data(10**self.orderOfMagnitude, mathtext=True)
+                    sciNotStr = r'{\times}'+self.format_data(10**self.orderOfMagnitude)
                 else:
-                    sciNotStr = 'x1e%+d'% self.orderOfMagnitude
-            if self._useMathText or self._usetex: return ''.join(('$',sciNotStr,offsetStr,'$'))
-            else: return ''.join((sciNotStr,offsetStr))
+                    sciNotStr = u'\xd7'+'1e%d'% self.orderOfMagnitude
+            if self._useMathText:
+                return ''.join(('$\mathdefault{',sciNotStr,offsetStr,'}$'))
+            elif self._usetex:
+                return ''.join(('$',sciNotStr,offsetStr,'$'))
+            else:
+                return ''.join((sciNotStr,offsetStr))
         else: return ''
 
     def set_locs(self, locs):
@@ -354,16 +357,16 @@ class ScalarFormatter(Formatter):
         if locs is None or not len(locs) or range == 0:
             self.offset = 0
             return
-        ave_loc = mean(locs)
+        ave_loc = npy.mean(locs)
         if ave_loc: # dont want to take log10(0)
-            ave_oom = math.floor(math.log10(mean(absolute(locs))))
+            ave_oom = math.floor(math.log10(npy.mean(npy.absolute(locs))))
             range_oom = math.floor(math.log10(range))
 
-            if absolute(ave_oom-range_oom) >= 3: # four sig-figs
+            if npy.absolute(ave_oom-range_oom) >= 3: # four sig-figs
                 if ave_loc < 0:
-                    self.offset = math.ceil(amax(locs)/10**range_oom)*10**range_oom
+                    self.offset = math.ceil(npy.max(locs)/10**range_oom)*10**range_oom
                 else:
-                    self.offset = math.floor(amin(locs)/10**(range_oom))*10**(range_oom)
+                    self.offset = math.floor(npy.min(locs)/10**(range_oom))*10**(range_oom)
             else: self.offset = 0
 
     def _set_orderOfMagnitude(self,range):
@@ -372,7 +375,7 @@ class ScalarFormatter(Formatter):
         if not self._scientific:
             self.orderOfMagnitude = 0
             return
-        locs = absolute(self.locs)
+        locs = npy.absolute(self.locs)
         if self.offset: oom = math.floor(math.log10(range))
         else:
             if locs[0] > locs[-1]: val = locs[0]
@@ -390,39 +393,41 @@ class ScalarFormatter(Formatter):
         # set the format string to format all the ticklabels
         # The floating point black magic (adding 1e-15 and formatting
         # to 8 digits) may warrant review and cleanup.
-        locs = (array(self.locs)-self.offset) / 10**self.orderOfMagnitude+1e-15
+        locs = (npy.asarray(self.locs)-self.offset) / 10**self.orderOfMagnitude+1e-15
         sigfigs = [len(str('%1.8f'% loc).split('.')[1].rstrip('0')) \
                    for loc in locs]
         sigfigs.sort()
         self.format = '%1.' + str(sigfigs[-1]) + 'f'
-        if self._usetex or self._useMathText: self.format = '$%s$'%self.format
+        if self._usetex:
+            self.format = '$%s$' % self.format
+        elif self._useMathText:
+            self.format = '$\mathdefault{%s}$' % self.format
 
     def pprint_val(self, x):
         xp = (x-self.offset)/10**self.orderOfMagnitude
-        if absolute(xp) < 1e-8: xp = 0
+        if npy.absolute(xp) < 1e-8: xp = 0
         return self.format % xp
 
-    def _formatSciNotation(self,s, mathtext=False):
+    def _formatSciNotation(self, s):
         # transform 1e+004 into 1e4, for example
         tup = s.split('e')
         try:
-            mantissa = tup[0].rstrip('0').rstrip('.')
+            significand = tup[0].rstrip('0').rstrip('.')
             sign = tup[1][0].replace('+', '')
             exponent = tup[1][1:].lstrip('0')
-            if mathtext:
-                if self._usetex:
-                    if mantissa=='1':
-                        return r'10^{%s%s}'%(sign, exponent)
-                    else:
-                        return r'%s{\times}10^{%s%s}'%(mantissa, sign, exponent)
+            if self._useMathText or self._usetex:
+                if significand == '1':
+                    # reformat 1x10^y as 10^y
+                    significand = ''
+                if exponent:
+                    exponent = '10^{%s%s}'%(sign, exponent)
+                if significand and exponent:
+                    return r'%s{\times}%s'%(significand, exponent)
                 else:
-                    if mantissa=='1':
-                        return r'10^{%s%s}'%(sign, exponent)
-                    else:
-                        return r'%s{\times}10^{%s%s}'%(mantissa, sign, exponent)
+                    return r'%s%s'%(significand, exponent)
             else:
-                return ('%se%s%s' %(mantissa, sign, exponent)).rstrip('e')
-        except IndexError,msg:
+                return ('%se%s%s' %(significand, sign, exponent)).rstrip('e')
+        except IndexError, msg:
             return s
 
 
@@ -467,7 +472,7 @@ class LogFormatter(Formatter):
 
     def format_data(self,value):
         self.labelOnlyBase = False
-        value = strip_math(self.__call__(value))
+        value = cbook.strip_math(self.__call__(value))
         self.labelOnlyBase = True
         return value
 
@@ -546,11 +551,13 @@ class LogFormatterMathtext(LogFormatter):
         elif not isDecade:
             if usetex:
                 s = r'$%d^{%.2f}$'% (b, fx)
-            else: s = '$%d^{%.2f}$'% (b, fx)
+            else:
+                s = '$\mathdefault{%d^{%.2f}}$'% (b, fx)
         else:
             if usetex:
                 s = r'$%d^{%d}$'% (b, self.nearest_long(fx))
-            else: s = r'$%d^{%d}$'% (b, self.nearest_long(fx))
+            else:
+                s = r'$\mathdefault{%d^{%d}}$'% (b, self.nearest_long(fx))
 
         return s
 
@@ -573,7 +580,7 @@ class Locator(TickHelper):
     def autoscale(self):
         'autoscale the view limits'
         self.verify_intervals()
-        return nonsingular(*self.dataInterval.get_bounds())
+        return mtrans.nonsingular(*self.dataInterval.get_bounds())
 
     def pan(self, numsteps):
         'Pan numticks (can be positive or negative)'
@@ -614,7 +621,7 @@ class IndexLocator(Locator):
     def __call__(self):
         'Return the locations of the ticks'
         dmin, dmax = self.dataInterval.get_bounds()
-        return arange(dmin + self.offset, dmax+1, self._base)
+        return npy.arange(dmin + self.offset, dmax+1, self._base)
 
 
 class FixedLocator(Locator):
@@ -687,7 +694,7 @@ class LinearLocator(Locator):
 
 
         if self.numticks==0: return []
-        ticklocs = linspace(vmin, vmax, self.numticks)
+        ticklocs = npy.linspace(vmin, vmax, self.numticks)
 
         return ticklocs
 
@@ -715,7 +722,7 @@ class LinearLocator(Locator):
         vmin = math.floor(scale*vmin)/scale
         vmax = math.ceil(scale*vmax)/scale
 
-        return nonsingular(vmin, vmax)
+        return mtrans.nonsingular(vmin, vmax)
 
 
 def closeto(x,y):
@@ -779,9 +786,9 @@ class MultipleLocator(Locator):
         if vmax<vmin:
             vmin, vmax = vmax, vmin
         vmin = self._base.ge(vmin)
-
-        locs =  frange(vmin, vmax+0.001*self._base.get_base(), self._base.get_base())
-
+        base = self._base.get_base()
+        n = (vmax - vmin + 0.001*base)//base
+        locs = vmin + npy.arange(n+1) * base
         return locs
 
     def autoscale(self):
@@ -799,7 +806,7 @@ class MultipleLocator(Locator):
             vmin -=1
             vmax +=1
 
-        return nonsingular(vmin, vmax)
+        return mtrans.nonsingular(vmin, vmax)
 
 def scale_range(vmin, vmax, n = 1, threshold=100):
     dv = abs(vmax - vmin)
@@ -861,20 +868,20 @@ class MaxNLocator(Locator):
         if self._trim:
             extra_bins = int(divmod((best_vmax - vmax), step)[0])
             nbins -= extra_bins
-        return (arange(nbins+1) * step + best_vmin + offset)
+        return (npy.arange(nbins+1) * step + best_vmin + offset)
 
 
     def __call__(self):
         self.verify_intervals()
         vmin, vmax = self.viewInterval.get_bounds()
-        vmin, vmax = nonsingular(vmin, vmax, expander = 0.05)
+        vmin, vmax = mtrans.nonsingular(vmin, vmax, expander = 0.05)
         return self.bin_boundaries(vmin, vmax)
 
     def autoscale(self):
         self.verify_intervals()
         dmin, dmax = self.dataInterval.get_bounds()
-        dmin, dmax = nonsingular(dmin, dmax, expander = 0.05)
-        return take(self.bin_boundaries(dmin, dmax), [0,-1])
+        dmin, dmax = mtrans.nonsingular(dmin, dmax, expander = 0.05)
+        return npy.take(self.bin_boundaries(dmin, dmax), [0,-1])
 
 
 def decade_down(x, base=10):
@@ -918,7 +925,7 @@ class LogLocator(Locator):
         if subs is None:
             self._subs = None  # autosub
         else:
-            self._subs = array(subs)+0.0
+            self._subs = npy.asarray(subs)+0.0
 
     def _set_numticks(self):
         self.numticks = 15  # todo; be smart here; this is just for dev
@@ -939,9 +946,9 @@ class LogLocator(Locator):
         numdec = math.floor(vmax)-math.ceil(vmin)
 
         if self._subs is None: # autosub
-            if numdec>10: subs = array([1.0])
-            elif numdec>6: subs = arange(2.0, b, 2.0)
-            else: subs = arange(2.0, b)
+            if numdec>10: subs = npy.array([1.0])
+            elif numdec>6: subs = npy.arange(2.0, b, 2.0)
+            else: subs = npy.arange(2.0, b)
         else:
             subs = self._subs
 
@@ -949,10 +956,11 @@ class LogLocator(Locator):
         while numdec/stride+1 > self.numticks:
             stride += 1
 
-        for decadeStart in b**arange(math.floor(vmin),math.ceil(vmax)+stride, stride):
+        for decadeStart in b**npy.arange(math.floor(vmin),
+                                         math.ceil(vmax)+stride, stride):
             ticklocs.extend( subs*decadeStart )
 
-        return array(ticklocs)
+        return npy.array(ticklocs)
 
     def autoscale(self):
         'Try to choose the view limits intelligently'
@@ -973,7 +981,7 @@ class LogLocator(Locator):
         if vmin==vmax:
             vmin = decade_down(vmin,self._base)
             vmax = decade_up(vmax,self._base)
-        return nonsingular(vmin, vmax)
+        return mtrans.nonsingular(vmin, vmax)
 
 class AutoLocator(MaxNLocator):
     def __init__(self):
