@@ -54,6 +54,26 @@ typedef agg::amask_no_clip_gray8 alpha_mask_type;
 typedef agg::renderer_base<agg::pixfmt_gray8> renderer_base_alpha_mask_type;
 typedef agg::renderer_scanline_aa_solid<renderer_base_alpha_mask_type> renderer_alpha_mask_type;
 
+struct SnapData {
+  SnapData(const bool& newpoint, const float& xsnap, const float& ysnap) :
+    newpoint(newpoint), xsnap(xsnap), ysnap(ysnap) {}
+  bool newpoint;
+  float xsnap, ysnap;
+};
+
+class SafeSnap {
+  // snap to pixel center, avoiding 0 path length rounding errors.
+public:
+  SafeSnap() : first(true), xsnap(0.0), lastx(0.0), lastxsnap(0.0),
+	       ysnap(0.0), lasty(0.0), lastysnap(0.0)  {}
+  SnapData snap (const float& x, const float& y);
+      
+private:
+  bool first;
+  float xsnap, lastx, lastxsnap, ysnap, lasty, lastysnap;
+};
+
+
 // a helper class to pass agg::buffer objects around.  agg::buffer is
 // a class in the swig wrapper
 class BufferRegion : public Py::PythonExtension<BufferRegion> {
@@ -146,7 +166,7 @@ public:
   //Py::Object _draw_markers_nocache(const Py::Tuple & args);
   //Py::Object _draw_markers_cache(const Py::Tuple & args);
   Py::Object draw_markers(const Py::Tuple & args);
-  Py::Object draw_text(const Py::Tuple & args);
+  Py::Object draw_text_image(const Py::Tuple & args);
   Py::Object draw_image(const Py::Tuple & args);
 
   Py::Object write_rgba(const Py::Tuple & args);
@@ -195,8 +215,8 @@ protected:
   agg::rect bbox_to_rect( const Py::Object& o);
   double points_to_pixels( const Py::Object& points);
   double points_to_pixels_snapto( const Py::Object& points);
-  void DrawQuadMesh(int, int, const agg::rgba8[], const double[], const double[]);
-  void DrawQuadMeshEdges(int, int, const agg::rgba8[], const double[], const double[]);
+  void DrawQuadMesh(int, int, void* colors, const double[], const double[]);
+  void DrawQuadMeshEdges(int, int, const double[], const double[]);
   int intersectCheck(double, double, double, double, double, int*);
   int inPolygon(int, const double[4], const double[4], int[4]);
   void set_clip_from_bbox(const Py::Object& o);
